@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 /**
  * SECTION:element-jpegenc
@@ -41,7 +41,6 @@
 #include "gstjpeg.h"
 #include <gst/video/video.h>
 #include <gst/video/gstvideometa.h>
-#include <gst/base/base.h>
 
 /* experimental */
 /* setting smoothig seems to have no effect in libjepeg
@@ -54,7 +53,6 @@ GST_DEBUG_CATEGORY_STATIC (jpegenc_debug);
 #define JPEG_DEFAULT_QUALITY 85
 #define JPEG_DEFAULT_SMOOTHING 0
 #define JPEG_DEFAULT_IDCT_METHOD	JDCT_FASTEST
-#define JPEG_DEFAULT_SNAPSHOT		FALSE
 
 /* JpegEnc signals and args */
 enum
@@ -68,10 +66,10 @@ enum
   PROP_0,
   PROP_QUALITY,
   PROP_SMOOTHING,
-  PROP_IDCT_METHOD,
-  PROP_SNAPSHOT
+  PROP_IDCT_METHOD
 };
 
+static void gst_jpegenc_reset (GstJpegEnc * enc);
 static void gst_jpegenc_finalize (GObject * object);
 
 static void gst_jpegenc_resync (GstJpegEnc * jpegenc);
@@ -100,8 +98,8 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE
-        ("{ I420, YV12, YUY2, UYVY, Y41B, Y42B, YVYU, Y444, NV21, "
-         "NV12, RGB, BGR, RGBx, xRGB, BGRx, xBGR, GRAY8 }"))
+        ("{ I420, YV12, YUY2, UYVY, Y41B, Y42B, YVYU, Y444, "
+         "RGB, BGR, RGBx, xRGB, BGRx, xBGR, GRAY8 }"))
     );
 /* *INDENT-ON* */
 
@@ -111,9 +109,7 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("image/jpeg, "
         "width = (int) [ 16, 65535 ], "
-        "height = (int) [ 16, 65535 ], "
-        "framerate = (fraction) [ 0/1, MAX ], "
-        "sof-marker = (int) { 0, 1, 2, 4, 9 }")
+        "height = (int) [ 16, 65535 ], " "framerate = (fraction) [ 0/1, MAX ]")
     );
 
 static void
@@ -136,8 +132,7 @@ gst_jpegenc_class_init (GstJpegEncClass * klass)
   g_object_class_install_property (gobject_class, PROP_QUALITY,
       g_param_spec_int ("quality", "Quality", "Quality of encoding",
           0, 100, JPEG_DEFAULT_QUALITY,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS |
-          GST_PARAM_MUTABLE_PLAYING));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 #ifdef ENABLE_SMOOTHING
   /* disabled, since it doesn't seem to work */
@@ -152,26 +147,14 @@ gst_jpegenc_class_init (GstJpegEncClass * klass)
           "The IDCT algorithm to use", GST_TYPE_IDCT_METHOD,
           JPEG_DEFAULT_IDCT_METHOD,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  /**
-   * GstJpegEnc:snapshot:
-   *
-   * Send EOS after encoding a frame, useful for snapshots.
-   *
-   * Since: 1.14
-   */
-  g_object_class_install_property (gobject_class, PROP_SNAPSHOT,
-      g_param_spec_boolean ("snapshot", "Snapshot",
-          "Send EOS after encoding a frame, useful for snapshots",
-          JPEG_DEFAULT_SNAPSHOT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_jpegenc_sink_pad_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_jpegenc_src_pad_template);
 
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_jpegenc_sink_pad_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_jpegenc_src_pad_template));
   gst_element_class_set_static_metadata (element_class, "JPEG image encoder",
-      "Codec/Encoder/Image", "Encode images in JPEG format",
-      "Wim Taymans <wim.taymans@tvd.be>");
+      "Codec/Encoder/Image",
+      "Encode images in JPEG format", "Wim Taymans <wim.taymans@tvd.be>");
 
   venc_class->start = gst_jpegenc_start;
   venc_class->stop = gst_jpegenc_stop;
@@ -245,40 +228,16 @@ gst_jpegenc_term_destination (j_compress_ptr cinfo)
 {
   GstBuffer *outbuf;
   GstJpegEnc *jpegenc = (GstJpegEnc *) (cinfo->client_data);
-  gsize memory_size = jpegenc->output_map.size - jpegenc->jdest.free_in_buffer;
-  GstByteReader reader =
-      GST_BYTE_READER_INIT (jpegenc->output_map.data, memory_size);
-  guint16 marker;
-  gint sof_marker = -1;
 
   GST_DEBUG_OBJECT (jpegenc, "gst_jpegenc_chain: term_source");
 
-  /* Find the SOF marker */
-  while (gst_byte_reader_get_uint16_be (&reader, &marker)) {
-    /* SOF marker */
-    if (marker >> 4 == 0x0ffc) {
-      sof_marker = marker & 0x4;
-      break;
-    }
-  }
-
   gst_memory_unmap (jpegenc->output_mem, &jpegenc->output_map);
   /* Trim the buffer size. we will push it in the chain function */
-  gst_memory_resize (jpegenc->output_mem, 0, memory_size);
+  gst_memory_resize (jpegenc->output_mem, 0,
+      jpegenc->output_map.size - jpegenc->jdest.free_in_buffer);
   jpegenc->output_map.data = NULL;
   jpegenc->output_map.size = 0;
 
-  if (jpegenc->sof_marker != sof_marker || jpegenc->input_caps_changed) {
-    GstVideoCodecState *output;
-    output =
-        gst_video_encoder_set_output_state (GST_VIDEO_ENCODER (jpegenc),
-        gst_caps_new_simple ("image/jpeg", "sof-marker", G_TYPE_INT, sof_marker,
-            NULL), jpegenc->input_state);
-    gst_video_codec_state_unref (output);
-    jpegenc->sof_marker = sof_marker;
-    jpegenc->input_caps_changed = FALSE;
-  }
-
   outbuf = gst_buffer_new ();
   gst_buffer_copy_into (outbuf, jpegenc->current_frame->input_buffer,
       GST_BUFFER_COPY_METADATA, 0, -1);
@@ -291,7 +250,7 @@ gst_jpegenc_term_destination (j_compress_ptr cinfo)
 
   GST_VIDEO_CODEC_FRAME_SET_SYNC_POINT (jpegenc->current_frame);
 
-  jpegenc->res = gst_video_encoder_finish_frame (GST_VIDEO_ENCODER (jpegenc),
+  gst_video_encoder_finish_frame (GST_VIDEO_ENCODER (jpegenc),
       jpegenc->current_frame);
   jpegenc->current_frame = NULL;
 }
@@ -299,8 +258,6 @@ gst_jpegenc_term_destination (j_compress_ptr cinfo)
 static void
 gst_jpegenc_init (GstJpegEnc * jpegenc)
 {
-  GST_PAD_SET_ACCEPT_TEMPLATE (GST_VIDEO_ENCODER_SINK_PAD (jpegenc));
-
   /* setup jpeglib */
   memset (&jpegenc->cinfo, 0, sizeof (jpegenc->cinfo));
   memset (&jpegenc->jerr, 0, sizeof (jpegenc->jerr));
@@ -317,7 +274,27 @@ gst_jpegenc_init (GstJpegEnc * jpegenc)
   jpegenc->quality = JPEG_DEFAULT_QUALITY;
   jpegenc->smoothing = JPEG_DEFAULT_SMOOTHING;
   jpegenc->idct_method = JPEG_DEFAULT_IDCT_METHOD;
-  jpegenc->snapshot = JPEG_DEFAULT_SNAPSHOT;
+
+  gst_jpegenc_reset (jpegenc);
+}
+
+static void
+gst_jpegenc_reset (GstJpegEnc * enc)
+{
+  gint i, j;
+
+  g_free (enc->line[0]);
+  g_free (enc->line[1]);
+  g_free (enc->line[2]);
+  enc->line[0] = NULL;
+  enc->line[1] = NULL;
+  enc->line[2] = NULL;
+  for (i = 0; i < 3; i++) {
+    for (j = 0; j < 4 * DCTSIZE; j++) {
+      g_free (enc->row[i][j]);
+      enc->row[i][j] = NULL;
+    }
+  }
 }
 
 static void
@@ -339,6 +316,7 @@ gst_jpegenc_set_format (GstVideoEncoder * encoder, GstVideoCodecState * state)
   GstJpegEnc *enc = GST_JPEGENC (encoder);
   gint i;
   GstVideoInfo *info = &state->info;
+  GstVideoCodecState *output;
 
   if (enc->input_state)
     gst_video_codec_state_unref (enc->input_state);
@@ -377,7 +355,11 @@ gst_jpegenc_set_format (GstVideoEncoder * encoder, GstVideoCodecState * state)
   }
   enc->planar = (enc->inc[0] == 1 && enc->inc[1] == 1 && enc->inc[2] == 1);
 
-  enc->input_caps_changed = TRUE;
+  output =
+      gst_video_encoder_set_output_state (encoder,
+      gst_caps_new_empty_simple ("image/jpeg"), state);
+  gst_video_codec_state_unref (output);
+
   gst_jpegenc_resync (enc);
 
   return TRUE;
@@ -481,7 +463,6 @@ gst_jpegenc_handle_frame (GstVideoEncoder * encoder, GstVideoCodecFrame * frame)
         i) * stride[i];
   }
 
-  jpegenc->res = GST_FLOW_OK;
   jpegenc->output_mem = gst_allocator_alloc (NULL, jpegenc->bufsize, &params);
   gst_memory_map (jpegenc->output_mem, &jpegenc->output_map, GST_MAP_READWRITE);
 
@@ -492,13 +473,9 @@ gst_jpegenc_handle_frame (GstVideoEncoder * encoder, GstVideoCodecFrame * frame)
 #if JPEG_LIB_VERSION >= 70
   jpegenc->cinfo.do_fancy_downsampling = FALSE;
 #endif
-
-  GST_OBJECT_LOCK (jpegenc);
   jpegenc->cinfo.smoothing_factor = jpegenc->smoothing;
   jpegenc->cinfo.dct_method = jpegenc->idct_method;
   jpeg_set_quality (&jpegenc->cinfo, jpegenc->quality, TRUE);
-  GST_OBJECT_UNLOCK (jpegenc);
-
   jpeg_start_compress (&jpegenc->cinfo, TRUE);
 
   GST_LOG_OBJECT (jpegenc, "compressing");
@@ -543,12 +520,13 @@ gst_jpegenc_handle_frame (GstVideoEncoder * encoder, GstVideoCodecFrame * frame)
   jpeg_finish_compress (&jpegenc->cinfo);
   GST_LOG_OBJECT (jpegenc, "compressing done");
 
-  return (jpegenc->snapshot) ? GST_FLOW_EOS : jpegenc->res;
+  return GST_FLOW_OK;
 
 invalid_frame:
   {
     GST_WARNING_OBJECT (jpegenc, "invalid frame received");
-    return gst_video_encoder_finish_frame (encoder, frame);
+    gst_video_encoder_finish_frame (encoder, frame);
+    return GST_FLOW_OK;
   }
 }
 
@@ -581,9 +559,6 @@ gst_jpegenc_set_property (GObject * object, guint prop_id,
     case PROP_IDCT_METHOD:
       jpegenc->idct_method = g_value_get_enum (value);
       break;
-    case PROP_SNAPSHOT:
-      jpegenc->snapshot = g_value_get_boolean (value);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -612,9 +587,6 @@ gst_jpegenc_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_IDCT_METHOD:
       g_value_set_enum (value, jpegenc->idct_method);
       break;
-    case PROP_SNAPSHOT:
-      g_value_set_boolean (value, jpegenc->snapshot);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -631,7 +603,6 @@ gst_jpegenc_start (GstVideoEncoder * benc)
   enc->line[0] = NULL;
   enc->line[1] = NULL;
   enc->line[2] = NULL;
-  enc->sof_marker = -1;
 
   return TRUE;
 }
@@ -640,20 +611,8 @@ static gboolean
 gst_jpegenc_stop (GstVideoEncoder * benc)
 {
   GstJpegEnc *enc = (GstJpegEnc *) benc;
-  gint i, j;
 
-  g_free (enc->line[0]);
-  g_free (enc->line[1]);
-  g_free (enc->line[2]);
-  enc->line[0] = NULL;
-  enc->line[1] = NULL;
-  enc->line[2] = NULL;
-  for (i = 0; i < 3; i++) {
-    for (j = 0; j < 4 * DCTSIZE; j++) {
-      g_free (enc->row[i][j]);
-      enc->row[i][j] = NULL;
-    }
-  }
+  gst_jpegenc_reset (enc);
 
   return TRUE;
 }
