@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 
@@ -56,9 +56,6 @@ struct _GstJpegEnc
   GstVideoCodecState *input_state;
   GstVideoFrame current_vframe;
   GstVideoCodecFrame *current_frame;
-  GstFlowReturn res;
-
-  gboolean input_caps_changed;
 
   guint channels;
 
@@ -70,7 +67,6 @@ struct _GstJpegEnc
   gint h_max_samp;
   gint v_max_samp;
   gboolean planar;
-  gint sof_marker;
   /* the video buffer */
   gint bufsize;
   /* the jpeg line buffer */
@@ -86,7 +82,6 @@ struct _GstJpegEnc
   gint quality;
   gint smoothing;
   gint idct_method;
-  gboolean snapshot;
 
   GstMemory *output_mem;
   GstMapInfo output_map;
