@@ -15,23 +15,23 @@
  *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with gst-pulse; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  *  USA.
  */
 
 /**
  * SECTION:element-pulsesrc
- * @title: pulsesrc
  * @see_also: pulsesink
  *
  * This element captures audio from a
- * [PulseAudio sound server](http://www.pulseaudio.org).
+ * <ulink href="http://www.pulseaudio.org">PulseAudio sound server</ulink>.
  *
- * ## Example pipelines
+ * <refsect2>
+ * <title>Example pipelines</title>
  * |[
  * gst-launch-1.0 -v pulsesrc ! audioconvert ! vorbisenc ! oggmux ! filesink location=alsasrc.ogg
  * ]| Record from a sound card using pulseaudio and encode to Ogg/Vorbis.
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -43,9 +43,8 @@
 
 #include <gst/base/gstbasesrc.h>
 #include <gst/gsttaglist.h>
-#include <gst/audio/audio.h>
+#include <gst/audio/streamvolume.h>
 
-#include "gstpulseelements.h"
 #include "pulsesrc.h"
 #include "pulseutil.h"
 
@@ -54,23 +53,18 @@ GST_DEBUG_CATEGORY_EXTERN (pulse_debug);
 
 #define DEFAULT_SERVER            NULL
 #define DEFAULT_DEVICE            NULL
-#define DEFAULT_CURRENT_DEVICE    NULL
 #define DEFAULT_DEVICE_NAME       NULL
 
 #define DEFAULT_VOLUME          1.0
 #define DEFAULT_MUTE            FALSE
 #define MAX_VOLUME              10.0
 
-/* See the pulsesink code for notes on how we interact with the PA mainloop
- * thread. */
-
 enum
 {
   PROP_0,
   PROP_SERVER,
   PROP_DEVICE,
   PROP_DEVICE_NAME,
-  PROP_CURRENT_DEVICE,
   PROP_CLIENT_NAME,
   PROP_STREAM_PROPERTIES,
   PROP_SOURCE_OUTPUT_INDEX,
@@ -113,11 +107,31 @@ static GstStateChangeReturn gst_pulsesrc_change_state (GstElement *
 
 static GstClockTime gst_pulsesrc_get_time (GstClock * clock, GstPulseSrc * src);
 
+#if (G_BYTE_ORDER == G_LITTLE_ENDIAN)
+# define FORMATS "{ S16LE, S16BE, F32LE, F32BE, S32LE, S32BE, U8 }"
+#else
+# define FORMATS "{ S16BE, S16LE, F32BE, F32LE, S32BE, S32LE, U8 }"
+#endif
+
+static GstStaticPadTemplate pad_template = GST_STATIC_PAD_TEMPLATE ("src",
+    GST_PAD_SRC,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS ("audio/x-raw, "
+        "format = (string) " FORMATS ", "
+        "layout = (string) interleaved, "
+        "rate = (int) [ 1, MAX ], "
+        "channels = (int) [ 1, 32 ];"
+        "audio/x-alaw, "
+        "rate = (int) [ 1, MAX], "
+        "channels = (int) [ 1, 32 ];"
+        "audio/x-mulaw, "
+        "rate = (int) [ 1, MAX], " "channels = (int) [ 1, 32 ]")
+    );
+
+
 #define gst_pulsesrc_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstPulseSrc, gst_pulsesrc, GST_TYPE_AUDIO_SRC,
     G_IMPLEMENT_INTERFACE (GST_TYPE_STREAM_VOLUME, NULL));
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (pulsesrc, "pulsesrc",
-    GST_RANK_PRIMARY + 10, GST_TYPE_PULSESRC, pulse_element_init (plugin));
 
 static void
 gst_pulsesrc_class_init (GstPulseSrcClass * klass)
@@ -126,7 +140,6 @@ gst_pulsesrc_class_init (GstPulseSrcClass * klass)
   GstAudioSrcClass *gstaudiosrc_class = GST_AUDIO_SRC_CLASS (klass);
   GstBaseSrcClass *gstbasesrc_class = GST_BASE_SRC_CLASS (klass);
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
-  GstCaps *caps;
   gchar *clientname;
 
   gobject_class->finalize = gst_pulsesrc_finalize;
@@ -159,11 +172,6 @@ gst_pulsesrc_class_init (GstPulseSrcClass * klass)
           "The PulseAudio source device to connect to", DEFAULT_DEVICE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (gobject_class, PROP_CURRENT_DEVICE,
-      g_param_spec_string ("current-device", "Current Device",
-          "The current PulseAudio source device", DEFAULT_CURRENT_DEVICE,
-          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-
   g_object_class_install_property (gobject_class,
       PROP_DEVICE_NAME,
       g_param_spec_string ("device-name", "Device name",
@@ -185,10 +193,10 @@ gst_pulsesrc_class_init (GstPulseSrcClass * klass)
   g_free (clientname);
 
   /**
-   * GstPulseSrc:stream-properties:
+   * GstPulseSrc:stream-properties
    *
    * List of pulseaudio stream properties. A list of defined properties can be
-   * found in the [pulseaudio api docs](http://0pointer.de/lennart/projects/pulseaudio/doxygen/proplist_8h.html).
+   * found in the <ulink href="http://0pointer.de/lennart/projects/pulseaudio/doxygen/proplist_8h.html">pulseaudio api docs</ulink>.
    *
    * Below is an example for registering as a music application to pulseaudio.
    * |[
@@ -198,6 +206,8 @@ gst_pulsesrc_class_init (GstPulseSrcClass * klass)
    * g_object_set (pulse, "stream-properties", props, NULL);
    * gst_structure_free (props);
    * ]|
+   *
+   * Since: 0.10.26
    */
   g_object_class_install_property (gobject_class,
       PROP_STREAM_PROPERTIES,
@@ -205,9 +215,11 @@ gst_pulsesrc_class_init (GstPulseSrcClass * klass)
           "list of pulseaudio stream properties",
           GST_TYPE_STRUCTURE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
-   * GstPulseSrc:source-output-index:
+   * GstPulseSrc:source-output-index
    *
    * The index of the PulseAudio source output corresponding to this element.
+   *
+   * Since: 0.10.31
    */
   g_object_class_install_property (gobject_class,
       PROP_SOURCE_OUTPUT_INDEX,
@@ -220,14 +232,11 @@ gst_pulsesrc_class_init (GstPulseSrcClass * klass)
       "PulseAudio Audio Source",
       "Source/Audio",
       "Captures audio from a PulseAudio server", "Lennart Poettering");
-
-  caps = gst_pulse_fix_pcm_caps (gst_caps_from_string (_PULSE_CAPS_PCM));
   gst_element_class_add_pad_template (gstelement_class,
-      gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS, caps));
-  gst_caps_unref (caps);
+      gst_static_pad_template_get (&pad_template));
 
   /**
-   * GstPulseSrc:volume:
+   * GstPulseSrc:volume
    *
    * The volume of the record stream.
    */
@@ -238,7 +247,7 @@ gst_pulsesrc_class_init (GstPulseSrcClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstPulseSrc:mute:
+   * GstPulseSrc:mute
    *
    * Whether the stream is muted or not.
    */
@@ -281,6 +290,8 @@ gst_pulsesrc_init (GstPulseSrc * pulsesrc)
   pulsesrc->properties = NULL;
   pulsesrc->proplist = NULL;
 
+  pulsesrc->probe = gst_pulseprobe_new (G_OBJECT (pulsesrc), G_OBJECT_GET_CLASS (pulsesrc), PROP_DEVICE, pulsesrc->server, FALSE, TRUE);        /* FALSE for sinks, TRUE for sources */
+
   /* this should be the default but it isn't yet */
   gst_audio_base_src_set_slave_method (GST_AUDIO_BASE_SRC (pulsesrc),
       GST_AUDIO_BASE_SRC_SLAVE_SKEW);
@@ -337,13 +348,17 @@ gst_pulsesrc_finalize (GObject * object)
   g_free (pulsesrc->server);
   g_free (pulsesrc->device);
   g_free (pulsesrc->client_name);
-  g_free (pulsesrc->current_source_name);
 
   if (pulsesrc->properties)
     gst_structure_free (pulsesrc->properties);
   if (pulsesrc->proplist)
     pa_proplist_free (pulsesrc->proplist);
 
+  if (pulsesrc->probe) {
+    gst_pulseprobe_free (pulsesrc->probe);
+    pulsesrc->probe = NULL;
+  }
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -453,24 +468,17 @@ gst_pulsesrc_source_output_info_cb (pa_context * c,
   if (i->index == psrc->source_output_idx) {
     psrc->volume = pa_sw_volume_to_linear (pa_cvolume_max (&i->volume));
     psrc->mute = i->mute;
-    psrc->current_source_idx = i->source;
-
-    if (G_UNLIKELY (psrc->volume > MAX_VOLUME)) {
-      GST_WARNING_OBJECT (psrc, "Clipped volume from %f to %f",
-          psrc->volume, MAX_VOLUME);
-      psrc->volume = MAX_VOLUME;
-    }
   }
 
 done:
   pa_threaded_mainloop_signal (psrc->mainloop, 0);
 }
 
-static void
-gst_pulsesrc_get_source_output_info (GstPulseSrc * pulsesrc, gdouble * volume,
-    gboolean * mute)
+static gdouble
+gst_pulsesrc_get_stream_volume (GstPulseSrc * pulsesrc)
 {
   pa_operation *o = NULL;
+  gdouble v;
 
   if (!pulsesrc->mainloop)
     goto no_mainloop;
@@ -492,37 +500,33 @@ gst_pulsesrc_get_source_output_info (GstPulseSrc * pulsesrc, gdouble * volume,
   }
 
 unlock:
-
-  if (volume)
-    *volume = pulsesrc->volume;
-  if (mute)
-    *mute = pulsesrc->mute;
+  v = pulsesrc->volume;
 
   if (o)
     pa_operation_unref (o);
 
   pa_threaded_mainloop_unlock (pulsesrc->mainloop);
 
-  return;
+  if (v > MAX_VOLUME) {
+    GST_WARNING_OBJECT (pulsesrc, "Clipped volume from %f to %f", v,
+        MAX_VOLUME);
+    v = MAX_VOLUME;
+  }
+
+  return v;
 
   /* ERRORS */
 no_mainloop:
   {
+    v = pulsesrc->volume;
     GST_DEBUG_OBJECT (pulsesrc, "we have no mainloop");
-    if (volume)
-      *volume = pulsesrc->volume;
-    if (mute)
-      *mute = pulsesrc->mute;
-    return;
+    return v;
   }
 no_index:
   {
+    v = pulsesrc->volume;
     GST_DEBUG_OBJECT (pulsesrc, "we don't have a stream index");
-    if (volume)
-      *volume = pulsesrc->volume;
-    if (mute)
-      *mute = pulsesrc->mute;
-    return;
+    return v;
   }
 info_failed:
   {
@@ -533,34 +537,11 @@ info_failed:
   }
 }
 
-static void
-gst_pulsesrc_current_source_info_cb (pa_context * c, const pa_source_info * i,
-    int eol, void *userdata)
-{
-  GstPulseSrc *psrc;
-
-  psrc = GST_PULSESRC_CAST (userdata);
-
-  if (!i)
-    goto done;
-
-  /* If the index doesn't match our current stream,
-   * it implies we just recreated the stream (caps change)
-   */
-  if (i->index == psrc->current_source_idx) {
-    g_free (psrc->current_source_name);
-    psrc->current_source_name = g_strdup (i->name);
-  }
-
-done:
-  pa_threaded_mainloop_signal (psrc->mainloop, 0);
-}
-
-static gchar *
-gst_pulsesrc_get_current_device (GstPulseSrc * pulsesrc)
+static gboolean
+gst_pulsesrc_get_stream_mute (GstPulseSrc * pulsesrc)
 {
   pa_operation *o = NULL;
-  gchar *current_src;
+  gboolean mute;
 
   if (!pulsesrc->mainloop)
     goto no_mainloop;
@@ -568,13 +549,10 @@ gst_pulsesrc_get_current_device (GstPulseSrc * pulsesrc)
   if (pulsesrc->source_output_idx == PA_INVALID_INDEX)
     goto no_index;
 
-  gst_pulsesrc_get_source_output_info (pulsesrc, NULL, NULL);
-
   pa_threaded_mainloop_lock (pulsesrc->mainloop);
 
-
-  if (!(o = pa_context_get_source_info_by_index (pulsesrc->context,
-              pulsesrc->current_source_idx, gst_pulsesrc_current_source_info_cb,
+  if (!(o = pa_context_get_source_output_info (pulsesrc->context,
+              pulsesrc->source_output_idx, gst_pulsesrc_source_output_info_cb,
               pulsesrc)))
     goto info_failed;
 
@@ -585,26 +563,27 @@ gst_pulsesrc_get_current_device (GstPulseSrc * pulsesrc)
   }
 
 unlock:
-
-  current_src = g_strdup (pulsesrc->current_source_name);
+  mute = pulsesrc->mute;
 
   if (o)
     pa_operation_unref (o);
 
   pa_threaded_mainloop_unlock (pulsesrc->mainloop);
 
-  return current_src;
+  return mute;
 
   /* ERRORS */
 no_mainloop:
   {
+    mute = pulsesrc->mute;
     GST_DEBUG_OBJECT (pulsesrc, "we have no mainloop");
-    return NULL;
+    return mute;
   }
 no_index:
   {
+    mute = pulsesrc->mute;
     GST_DEBUG_OBJECT (pulsesrc, "we don't have a stream index");
-    return NULL;
+    return mute;
   }
 info_failed:
   {
@@ -624,7 +603,7 @@ gst_pulsesrc_set_stream_volume (GstPulseSrc * pulsesrc, gdouble volume)
   if (!pulsesrc->mainloop)
     goto no_mainloop;
 
-  if (pulsesrc->source_output_idx == PA_INVALID_INDEX)
+  if (!pulsesrc->source_output_idx)
     goto no_index;
 
   pa_threaded_mainloop_lock (pulsesrc->mainloop);
@@ -679,7 +658,7 @@ gst_pulsesrc_set_stream_mute (GstPulseSrc * pulsesrc, gboolean mute)
   if (!pulsesrc->mainloop)
     goto no_mainloop;
 
-  if (pulsesrc->source_output_idx == PA_INVALID_INDEX)
+  if (!pulsesrc->source_output_idx)
     goto no_index;
 
   pa_threaded_mainloop_lock (pulsesrc->mainloop);
@@ -724,55 +703,6 @@ mute_failed:
   }
 }
 
-static void
-gst_pulsesrc_set_stream_device (GstPulseSrc * pulsesrc, const gchar * device)
-{
-  pa_operation *o = NULL;
-
-  if (!pulsesrc->mainloop)
-    goto no_mainloop;
-
-  if (pulsesrc->source_output_idx == PA_INVALID_INDEX)
-    goto no_index;
-
-  pa_threaded_mainloop_lock (pulsesrc->mainloop);
-
-  GST_DEBUG_OBJECT (pulsesrc, "setting stream device to %s", device);
-
-  if (!(o = pa_context_move_source_output_by_name (pulsesrc->context,
-              pulsesrc->source_output_idx, device, NULL, NULL)))
-    goto move_failed;
-
-unlock:
-
-  if (o)
-    pa_operation_unref (o);
-
-  pa_threaded_mainloop_unlock (pulsesrc->mainloop);
-
-  return;
-
-  /* ERRORS */
-no_mainloop:
-  {
-    GST_DEBUG_OBJECT (pulsesrc, "we have no mainloop");
-    return;
-  }
-no_index:
-  {
-    GST_DEBUG_OBJECT (pulsesrc, "we don't have a stream index");
-    return;
-  }
-move_failed:
-  {
-    GST_ELEMENT_ERROR (pulsesrc, RESOURCE, FAILED,
-        ("pa_context_move_source_output_by_name(%s) failed: %s",
-            device, pa_strerror (pa_context_errno (pulsesrc->context))),
-        (NULL));
-    goto unlock;
-  }
-}
-
 static void
 gst_pulsesrc_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec)
@@ -784,11 +714,12 @@ gst_pulsesrc_set_property (GObject * object,
     case PROP_SERVER:
       g_free (pulsesrc->server);
       pulsesrc->server = g_value_dup_string (value);
+      if (pulsesrc->probe)
+        gst_pulseprobe_set_server (pulsesrc->probe, pulsesrc->server);
       break;
     case PROP_DEVICE:
       g_free (pulsesrc->device);
       pulsesrc->device = g_value_dup_string (value);
-      gst_pulsesrc_set_stream_device (pulsesrc, pulsesrc->device);
       break;
     case PROP_CLIENT_NAME:
       g_free (pulsesrc->client_name);
@@ -834,15 +765,6 @@ gst_pulsesrc_get_property (GObject * object,
     case PROP_DEVICE:
       g_value_set_string (value, pulsesrc->device);
       break;
-    case PROP_CURRENT_DEVICE:
-    {
-      gchar *current_device = gst_pulsesrc_get_current_device (pulsesrc);
-      if (current_device)
-        g_value_take_string (value, current_device);
-      else
-        g_value_set_string (value, "");
-      break;
-    }
     case PROP_DEVICE_NAME:
       g_value_take_string (value, gst_pulsesrc_device_description (pulsesrc));
       break;
@@ -856,19 +778,11 @@ gst_pulsesrc_get_property (GObject * object,
       g_value_set_uint (value, pulsesrc->source_output_idx);
       break;
     case PROP_VOLUME:
-    {
-      gdouble volume;
-      gst_pulsesrc_get_source_output_info (pulsesrc, &volume, NULL);
-      g_value_set_double (value, volume);
+      g_value_set_double (value, gst_pulsesrc_get_stream_volume (pulsesrc));
       break;
-    }
     case PROP_MUTE:
-    {
-      gboolean mute;
-      gst_pulsesrc_get_source_output_info (pulsesrc, NULL, &mute);
-      g_value_set_boolean (value, mute);
+      g_value_set_boolean (value, gst_pulsesrc_get_stream_mute (pulsesrc));
       break;
-    }
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1090,7 +1004,6 @@ gst_pulsesrc_read (GstAudioSrc * asrc, gpointer data, guint length,
   if (g_atomic_int_compare_and_exchange (&pulsesrc->notify, 1, 0)) {
     g_object_notify (G_OBJECT (pulsesrc), "volume");
     g_object_notify (G_OBJECT (pulsesrc), "mute");
-    g_object_notify (G_OBJECT (pulsesrc), "current-device");
   }
 
   pa_threaded_mainloop_lock (pulsesrc->mainloop);
@@ -1244,7 +1157,7 @@ gst_pulsesrc_create_stream (GstPulseSrc * pulsesrc, GstCaps ** caps,
   const pa_channel_map *m;
   GstStructure *s;
   gboolean need_channel_layout = FALSE;
-  GstAudioRingBufferSpec new_spec, *spec = NULL;
+  GstAudioRingBufferSpec new_spec, *spec;
   const gchar *name;
   int i;
 
@@ -1502,20 +1415,7 @@ gst_pulsesrc_prepare (GstAudioSrc * asrc, GstAudioRingBufferSpec * spec)
 
   pa_operation_unref (o);
 
-  /* There's a bit of a disconnect here between the audio ringbuffer and what
-   * PulseAudio provides. The audio ringbuffer provide a total of buffer_time
-   * worth of buffering, divided into segments of latency_time size. We're
-   * asking PulseAudio to provide a total latency of latency_time, which, with
-   * PA_STREAM_ADJUST_LATENCY, effectively sets itself up as a ringbuffer with
-   * one segment being the hardware buffer, and the other the software buffer.
-   * This segment size is returned as the fragsize.
-   *
-   * Since the two concepts don't map very well, what we do is keep segsize as
-   * it is (unless fragsize is even larger, in which case we use that). We'll
-   * get data from PulseAudio in smaller chunks than we want to pass on as an
-   * element, and we coalesce those chunks in the ringbuffer memory and pass it
-   * on in the expected chunk size. */
-  wanted.maxlength = spec->segsize * spec->segtotal;
+  wanted.maxlength = -1;
   wanted.tlength = -1;
   wanted.prebuf = 0;
   wanted.minreq = -1;
@@ -1588,18 +1488,12 @@ gst_pulsesrc_prepare (GstAudioSrc * asrc, GstAudioRingBufferSpec * spec)
   GST_INFO_OBJECT (pulsesrc, "fragsize:  %d (wanted %d)",
       actual->fragsize, wanted.fragsize);
 
-  if (actual->fragsize >= spec->segsize) {
+  if (actual->fragsize >= wanted.fragsize) {
     spec->segsize = actual->fragsize;
   } else {
-    /* fragsize is smaller than what we wanted, so let the read function
-     * coalesce the smaller chunks as they come in */
+    spec->segsize = actual->fragsize * (wanted.fragsize / actual->fragsize);
   }
-
-  /* Fix up the total ringbuffer size based on what we actually got */
   spec->segtotal = actual->maxlength / spec->segsize;
-  /* Don't buffer less than 2 segments as the ringbuffer can't deal with it */
-  if (spec->segtotal < 2)
-    spec->segtotal = 2;
 
   if (!pulsesrc->paused) {
     GST_DEBUG_OBJECT (pulsesrc, "uncorking because we are playing");
@@ -1860,9 +1754,6 @@ gst_pulsesrc_get_time (GstClock * clock, GstPulseSrc * src)
 {
   pa_usec_t time = 0;
 
-  if (src->mainloop == NULL)
-    goto out;
-
   pa_threaded_mainloop_lock (src->mainloop);
   if (!src->stream)
     goto unlock_and_out;
@@ -1881,6 +1772,5 @@ gst_pulsesrc_get_time (GstClock * clock, GstPulseSrc * src)
 unlock_and_out:
   pa_threaded_mainloop_unlock (src->mainloop);
 
-out:
   return time;
 }
