@@ -15,24 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
- */
-
-/**
- * SECTION:element-shout2send
- * @title: shout2send
- *
- * shout2send pushes a media stream to an Icecast server
- *
- * ## Example launch line
- * |[
- * gst-launch-1.0 uridecodebin uri=file:///path/to/audiofile ! audioconvert ! vorbisenc ! oggmux ! shout2send mount=/stream.ogg port=8000 username=source password=somepassword ip=server_IP_address_or_hostname
- * ]| This pipeline demuxes, decodes, re-encodes and re-muxes an audio
- * media file into oggvorbis and sends the resulting stream to an Icecast
- * server. Properties mount, port, username and password are all server-config
- * dependent.
- *
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -43,25 +27,22 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <glib/gi18n-lib.h>
-
-#ifndef HAVE_SHOUT_2_4_6_OR_NEWER
-#define shout_set_metadata_utf8 shout_set_metadata
-#endif
+#include "gst/gst-i18n-plugin.h"
 
 GST_DEBUG_CATEGORY_STATIC (shout2_debug);
 #define GST_CAT_DEFAULT shout2_debug
 
+
 enum
 {
-  SIGNAL_CONNECTION_PROBLEM,    /* FIXME 2.0: remove this */
+  SIGNAL_CONNECTION_PROBLEM,    /* 0.11 FIXME: remove this */
   LAST_SIGNAL
 };
 
 enum
 {
   ARG_0,
-  ARG_IP,                       /* the IP address or hostname of the server */
+  ARG_IP,                       /* the ip of the server */
   ARG_PORT,                     /* the encoder port number on the server */
   ARG_PASSWORD,                 /* the encoder password on the server */
   ARG_USERNAME,                 /* the encoder username on the server */
@@ -73,37 +54,31 @@ enum
   ARG_PROTOCOL,                 /* Protocol to connect with */
 
   ARG_MOUNT,                    /* mountpoint of stream (icecast only) */
-  ARG_URL,                      /* the stream's homepage URL */
-
-  ARG_TIMEOUT,                  /* The max amount of time to wait for
-                                   network activity */
-  ARG_SEND_TITLE_INFO,          /* If stream song title updates should be made */
-  ARG_USERAGENT                 /* User-Agent setting */
+  ARG_URL                       /* Url of stream (I'm guessing) */
 };
 
 #define DEFAULT_IP           "127.0.0.1"
 #define DEFAULT_PORT         8000
 #define DEFAULT_PASSWORD     "hackme"
 #define DEFAULT_USERNAME     "source"
-#define DEFAULT_PUBLIC       FALSE
+#define DEFAULT_PUBLIC     FALSE
 #define DEFAULT_STREAMNAME   ""
 #define DEFAULT_DESCRIPTION  ""
-#define DEFAULT_USERAGENT    "GStreamer " PACKAGE_VERSION
 #define DEFAULT_GENRE        ""
 #define DEFAULT_MOUNT        ""
 #define DEFAULT_URL          ""
 #define DEFAULT_PROTOCOL     SHOUT2SEND_PROTOCOL_HTTP
-#define DEFAULT_TIMEOUT      10000
-#define DEFAULT_SEND_TITLE_INFO TRUE
-
-#define SHOUT2SEND_CAPS "application/ogg; audio/ogg; video/ogg; "\
-    "audio/mpeg, mpegversion = (int) 1, layer = (int) [ 1, 3 ]; " \
-    "video/webm; audio/webm"
 
+#ifdef SHOUT_FORMAT_WEBM
+#define WEBM_CAPS "; video/webm"
+#else
+#define WEBM_CAPS ""
+#endif
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (SHOUT2SEND_CAPS));
+    GST_STATIC_CAPS ("application/ogg; audio/ogg; video/ogg; "
+        "audio/mpeg, mpegversion = (int) 1, layer = (int) [ 1, 3 ]" WEBM_CAPS));
 
 static void gst_shout2send_finalize (GstShout2send * shout2send);
 
@@ -149,15 +124,6 @@ gst_shout2send_protocol_get_type (void)
 #define gst_shout2send_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstShout2send, gst_shout2send, GST_TYPE_BASE_SINK,
     G_IMPLEMENT_INTERFACE (GST_TYPE_TAG_SETTER, NULL));
-#ifdef ENABLE_NLS
-#define _do_init \
-  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);\
-  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-#else /* ENABLE_NLS */
-#define _do_init
-#endif
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (shout2send, "shout2send", GST_RANK_NONE,
-    GST_TYPE_SHOUT2SEND, _do_init);
 
 static void
 gst_shout2send_class_init (GstShout2sendClass * klass)
@@ -165,7 +131,6 @@ gst_shout2send_class_init (GstShout2sendClass * klass)
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
   GstBaseSinkClass *gstbasesink_class;
-  GstPadTemplate *tmpl;
 
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
@@ -177,9 +142,8 @@ gst_shout2send_class_init (GstShout2sendClass * klass)
   gobject_class->get_property = gst_shout2send_get_property;
   gobject_class->finalize = (GObjectFinalizeFunc) gst_shout2send_finalize;
 
-  /* FIXME: 2.0 Should probably change this prop name to "server" */
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_IP,
-      g_param_spec_string ("ip", "ip", "IP address or hostname", DEFAULT_IP,
+      g_param_spec_string ("ip", "ip", "ip", DEFAULT_IP,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_PORT,
       g_param_spec_int ("port", "port", "port", 1, G_MAXUSHORT, DEFAULT_PORT,
@@ -216,32 +180,6 @@ gst_shout2send_class_init (GstShout2sendClass * klass)
           GST_TYPE_SHOUT_PROTOCOL, DEFAULT_PROTOCOL,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  /**
-   * GstShout2send:send-title-info
-   *
-   * Update stream metadata with song title and artist information
-   *
-   * Since: 1.22
-   **/
-
-  g_object_class_install_property (G_OBJECT_CLASS (klass),
-      ARG_SEND_TITLE_INFO,
-      g_param_spec_boolean ("send-title-info", "send-title-info",
-          "Update stream metadata with song title and artist information",
-          DEFAULT_SEND_TITLE_INFO, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
-  /**
-   * GstShout2send:user-agent
-   *
-   * User agent of the source
-   *
-   * Since: 1.22
-   **/
-
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_USERAGENT,
-      g_param_spec_string ("user-agent", "user-agent",
-          "User agent of the source", DEFAULT_USERAGENT,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /* icecast only */
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MOUNT,
@@ -249,19 +187,15 @@ gst_shout2send_class_init (GstShout2sendClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_URL,
-      g_param_spec_string ("url", "url", "the stream's homepage URL",
-          DEFAULT_URL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_TIMEOUT,
-      g_param_spec_uint ("timeout", "timeout",
-          "Max amount of time to wait for network activity, in milliseconds",
-          1, G_MAXUINT, DEFAULT_TIMEOUT,
+      g_param_spec_string ("url", "url", "url", DEFAULT_URL,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /* signals */
   gst_shout2send_signals[SIGNAL_CONNECTION_PROBLEM] =
       g_signal_new ("connection-problem", G_TYPE_FROM_CLASS (klass),
-      0, 0, NULL, NULL, NULL, G_TYPE_NONE, 1, G_TYPE_INT);
+      G_SIGNAL_RUN_CLEANUP, G_STRUCT_OFFSET (GstShout2sendClass,
+          connection_problem), NULL, NULL, g_cclosure_marshal_VOID__INT,
+      G_TYPE_NONE, 1, G_TYPE_INT);
 
   gstbasesink_class->start = GST_DEBUG_FUNCPTR (gst_shout2send_start);
   gstbasesink_class->stop = GST_DEBUG_FUNCPTR (gst_shout2send_stop);
@@ -272,8 +206,8 @@ gst_shout2send_class_init (GstShout2sendClass * klass)
   gstbasesink_class->event = GST_DEBUG_FUNCPTR (gst_shout2send_event);
   gstbasesink_class->set_caps = GST_DEBUG_FUNCPTR (gst_shout2send_setcaps);
 
-  tmpl = gst_static_pad_template_get (&sink_template);
-  gst_element_class_add_pad_template (gstelement_class, tmpl);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sink_template));
 
   gst_element_class_set_static_metadata (gstelement_class,
       "Icecast network sink",
@@ -283,8 +217,6 @@ gst_shout2send_class_init (GstShout2sendClass * klass)
       "Zaheer Abbas Merali <zaheerabbas at merali dot org>");
 
   GST_DEBUG_CATEGORY_INIT (shout2_debug, "shout2", 0, "shout2send element");
-
-  gst_type_mark_as_plugin_api (GST_TYPE_SHOUT_PROTOCOL, 0);
 }
 
 static void
@@ -292,7 +224,7 @@ gst_shout2send_init (GstShout2send * shout2send)
 {
   gst_base_sink_set_sync (GST_BASE_SINK (shout2send), FALSE);
 
-  shout2send->timer = gst_poll_new (TRUE);
+  shout2send->timer = gst_poll_new_timer ();
 
   shout2send->ip = g_strdup (DEFAULT_IP);
   shout2send->port = DEFAULT_PORT;
@@ -305,18 +237,14 @@ gst_shout2send_init (GstShout2send * shout2send)
   shout2send->url = g_strdup (DEFAULT_URL);
   shout2send->protocol = DEFAULT_PROTOCOL;
   shout2send->ispublic = DEFAULT_PUBLIC;
-  shout2send->timeout = DEFAULT_TIMEOUT;
 
-  shout2send->format = -1;
-  shout2send->usage = SHOUT_USAGE_UNKNOWN;
   shout2send->tags = gst_tag_list_new_empty ();
   shout2send->conn = NULL;
+  shout2send->audio_format = SHOUT_FORMAT_VORBIS;
   shout2send->connected = FALSE;
   shout2send->songmetadata = NULL;
   shout2send->songartist = NULL;
   shout2send->songtitle = NULL;
-  shout2send->send_title_info = DEFAULT_SEND_TITLE_INFO;
-  shout2send->user_agent = g_strdup (DEFAULT_USERAGENT);
 }
 
 static void
@@ -330,7 +258,6 @@ gst_shout2send_finalize (GstShout2send * shout2send)
   g_free (shout2send->genre);
   g_free (shout2send->mount);
   g_free (shout2send->url);
-  g_free (shout2send->user_agent);
 
   gst_tag_list_unref (shout2send->tags);
 
@@ -380,10 +307,6 @@ set_shout_metadata (const GstTagList * list, const gchar * tag,
   if (*shout_metadata != NULL)
     g_free (*shout_metadata);
 
-  if (!shout2send->send_title_info) {
-    *shout_metadata = NULL;
-    return;
-  }
 
   if (*song_title && *song_artist) {
     *shout_metadata = g_strdup_printf ("%s - %s", *song_artist, *song_title);
@@ -442,8 +365,8 @@ gst_shout2send_event (GstBaseSink * sink, GstEvent * event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_TAG:{
-      /* vorbis audio doesn't need metadata setting on the icecast level, only mp3 */
-      if (shout2send->tags && shout2send->format == SHOUT_FORMAT_MP3) {
+      /* vorbis audio doesnt need metadata setting on the icecast level, only mp3 */
+      if (shout2send->tags && shout2send->audio_format == SHOUT_FORMAT_MP3) {
         GstTagList *list;
 
         gst_event_parse_tag (event, &list);
@@ -456,21 +379,13 @@ gst_shout2send_event (GstBaseSink * sink, GstEvent * event)
             set_shout_metadata, shout2send);
         if (shout2send->songmetadata && shout2send->connected) {
           shout_metadata_t *pmetadata;
-          int shout_ret;
 
           GST_DEBUG_OBJECT (shout2send, "metadata now: %s",
               shout2send->songmetadata);
 
           pmetadata = shout_metadata_new ();
-          shout_ret =
-              shout_metadata_add (pmetadata, "song", shout2send->songmetadata);
-          if (shout_ret == SHOUTERR_SUCCESS) {
-            shout_ret = shout_set_metadata_utf8 (shout2send->conn, pmetadata);
-            if (shout_ret != SHOUTERR_SUCCESS) {
-              GST_WARNING_OBJECT (shout2send, "Failed to set metadata: %s",
-                  shout_get_error (shout2send->conn));
-            }
-          }
+          shout_metadata_add (pmetadata, "song", shout2send->songmetadata);
+          shout_set_metadata (shout2send->conn, pmetadata);
           shout_metadata_free (pmetadata);
         }
       }
@@ -489,27 +404,13 @@ gst_shout2send_event (GstBaseSink * sink, GstEvent * event)
   return ret;
 }
 
-static gboolean
-gst_shout2send_set_meta (GstShout2send * sink, const char *meta,
-    const char *val)
-{
-  GST_DEBUG_OBJECT (sink, "setting %s: %s", meta, val);
-
-  if (shout_set_meta (sink->conn, meta, val) == SHOUTERR_SUCCESS)
-    return TRUE;
-
-  GST_ELEMENT_ERROR (sink, LIBRARY, SETTINGS, (NULL),
-      ("Error setting %s: %s", meta, shout_get_error (sink->conn)));
-
-  return FALSE;
-}
-
 static gboolean
 gst_shout2send_start (GstBaseSink * basesink)
 {
   GstShout2send *sink = GST_SHOUT2SEND (basesink);
   const gchar *cur_prop;
   gshort proto = 3;
+  gchar *version_string;
 
   GST_DEBUG_OBJECT (sink, "starting");
 
@@ -532,8 +433,10 @@ gst_shout2send_start (GstBaseSink * basesink)
   if (shout_set_protocol (sink->conn, proto) != SHOUTERR_SUCCESS)
     goto set_failed;
 
+  /* --- FIXME: shout requires an ip, and fails if it is given a host. */
+  /* may want to put convert_to_ip(shout2send->ip) here */
   cur_prop = "ip";
-  GST_DEBUG_OBJECT (sink, "setting IP/hostname: %s", sink->ip);
+  GST_DEBUG_OBJECT (sink, "setting ip: %s", sink->ip);
   if (shout_set_host (sink->conn, sink->ip) != SHOUTERR_SUCCESS)
     goto set_failed;
 
@@ -553,15 +456,20 @@ gst_shout2send_start (GstBaseSink * basesink)
           (sink->ispublic ? 1 : 0)) != SHOUTERR_SUCCESS)
     goto set_failed;
 
-  if (!gst_shout2send_set_meta (sink, SHOUT_META_NAME, sink->streamname))
-    goto set_meta_failed;
+  cur_prop = "streamname";
+  GST_DEBUG_OBJECT (sink, "setting %s: %s", cur_prop, sink->streamname);
+  if (shout_set_name (sink->conn, sink->streamname) != SHOUTERR_SUCCESS)
+    goto set_failed;
 
-  if (!gst_shout2send_set_meta (sink, SHOUT_META_DESCRIPTION,
-          sink->description))
-    goto set_meta_failed;
+  cur_prop = "description";
+  GST_DEBUG_OBJECT (sink, "setting %s: %s", cur_prop, sink->description);
+  if (shout_set_description (sink->conn, sink->description) != SHOUTERR_SUCCESS)
+    goto set_failed;
 
-  if (!gst_shout2send_set_meta (sink, SHOUT_META_GENRE, sink->genre))
-    goto set_meta_failed;
+  cur_prop = "genre";
+  GST_DEBUG_OBJECT (sink, "setting %s: %s", cur_prop, sink->genre);
+  if (shout_set_genre (sink->conn, sink->genre) != SHOUTERR_SUCCESS)
+    goto set_failed;
 
   cur_prop = "mount";
   GST_DEBUG_OBJECT (sink, "setting %s: %s", cur_prop, sink->mount);
@@ -569,16 +477,19 @@ gst_shout2send_start (GstBaseSink * basesink)
     goto set_failed;
 
   cur_prop = "username";
-  GST_DEBUG_OBJECT (sink, "setting %s: %s", cur_prop, sink->username);
+  GST_DEBUG_OBJECT (sink, "setting %s: %s", cur_prop, "source");
   if (shout_set_user (sink->conn, sink->username) != SHOUTERR_SUCCESS)
     goto set_failed;
 
+  version_string = gst_version_string ();
   cur_prop = "agent";
-  GST_DEBUG_OBJECT (sink, "setting %s: %s", cur_prop, sink->user_agent);
-  if (shout_set_agent (sink->conn, sink->user_agent) != SHOUTERR_SUCCESS) {
+  GST_DEBUG_OBJECT (sink, "setting %s: %s", cur_prop, version_string);
+  if (shout_set_agent (sink->conn, version_string) != SHOUTERR_SUCCESS) {
+    g_free (version_string);
     goto set_failed;
   }
 
+  g_free (version_string);
   return TRUE;
 
 /* ERROR */
@@ -586,118 +497,50 @@ set_failed:
   {
     GST_ELEMENT_ERROR (sink, LIBRARY, SETTINGS, (NULL),
         ("Error setting %s: %s", cur_prop, shout_get_error (sink->conn)));
-    /* fallthrough */
-  }
-set_meta_failed:
-  {
-    shout_free (sink->conn);
-    sink->conn = NULL;
     return FALSE;
   }
 }
 
-static GstFlowReturn
+static gboolean
 gst_shout2send_connect (GstShout2send * sink)
 {
-  GstFlowReturn fret = GST_FLOW_OK;
-  gint ret;
-  GstClockTime start_ts;
-
-  GST_DEBUG_OBJECT (sink, "Connection format is: %d", sink->format);
-
-  if (sink->format == -1)
-    goto no_caps;
-
-  if (shout_set_nonblocking (sink->conn, 1) != SHOUTERR_SUCCESS)
-    goto could_not_set_nonblocking;
-
-  {
-#ifdef HAVE_SHOUT_2_4_6_OR_NEWER
-    ret =
-        shout_set_content_format (sink->conn, sink->format, sink->usage, NULL);
-#else
-    ret = shout_set_format (sink->conn, sink->format);
+  const char *format =
+      (sink->audio_format == SHOUT_FORMAT_VORBIS) ? "vorbis" :
+      ((sink->audio_format == SHOUT_FORMAT_MP3) ? "mp3" : "unknown");
+#ifdef SHOUT_FORMAT_WEBM
+  if (sink->audio_format == SHOUT_FORMAT_WEBM)
+    format = "webm";
 #endif
+  GST_DEBUG_OBJECT (sink, "Connection format is: %s", format);
 
-    if (ret != SHOUTERR_SUCCESS)
-      goto could_not_set_format;
-  }
+  if (shout_set_format (sink->conn, sink->audio_format) != SHOUTERR_SUCCESS)
+    goto could_not_set_format;
 
-  GST_DEBUG_OBJECT (sink, "connecting");
-
-  start_ts = gst_util_get_timestamp ();
-  ret = shout_open (sink->conn);
-
-  /* wait for connection or timeout */
-  /* starting with libshout 2.4.2, shout_open() has broken API + ABI and
-   * can also return SHOUTERR_RETRY (a new define) to mean "try again" */
-  while (ret == SHOUTERR_BUSY || ret == SHOUTERR_RETRY) {
-    if (gst_util_get_timestamp () - start_ts > sink->timeout * GST_MSECOND) {
-      goto connection_timeout;
-    }
-    if (gst_poll_wait (sink->timer, 10 * GST_MSECOND) == -1) {
-      GST_LOG_OBJECT (sink, "unlocked");
-
-      fret = gst_base_sink_wait_preroll (GST_BASE_SINK (sink));
-      if (fret != GST_FLOW_OK)
-        goto done;
-    }
-    ret = shout_get_connected (sink->conn);
-  }
-
-  if (ret != SHOUTERR_CONNECTED && ret != SHOUTERR_SUCCESS)
+  if (shout_open (sink->conn) != SHOUTERR_SUCCESS)
     goto could_not_connect;
 
   GST_DEBUG_OBJECT (sink, "connected to server");
   sink->connected = TRUE;
 
-  /* initialize sending rate monitoring */
-  sink->prev_queuelen = 0;
-  sink->data_sent = 0;
-  sink->stalled = TRUE;
-  sink->datasent_reset_ts = sink->stalled_ts = gst_util_get_timestamp ();
-
   /* let's set metadata */
   if (sink->songmetadata) {
     shout_metadata_t *pmetadata;
 
     GST_DEBUG_OBJECT (sink, "shout metadata now: %s", sink->songmetadata);
     pmetadata = shout_metadata_new ();
-    ret = shout_metadata_add (pmetadata, "song", sink->songmetadata);
-    if (ret == SHOUTERR_SUCCESS) {
-      ret = shout_set_metadata_utf8 (sink->conn, pmetadata);
-      if (ret != SHOUTERR_SUCCESS) {
-        GST_WARNING_OBJECT (sink, "Failed to set metadata: %s",
-            shout_get_error (sink->conn));
-      }
-    }
+    shout_metadata_add (pmetadata, "song", sink->songmetadata);
+    shout_set_metadata (sink->conn, pmetadata);
     shout_metadata_free (pmetadata);
   }
 
-done:
-  return fret;
+  return TRUE;
 
 /* ERRORS */
-no_caps:
-  {
-    GST_ELEMENT_ERROR (sink, CORE, NEGOTIATION, (NULL),
-        ("No input caps received."));
-    return GST_FLOW_NOT_NEGOTIATED;
-  }
-
-could_not_set_nonblocking:
-  {
-    GST_ELEMENT_ERROR (sink, LIBRARY, SETTINGS, (NULL),
-        ("Error configuring libshout to use non-blocking i/o: %s",
-            shout_get_error (sink->conn)));
-    return GST_FLOW_ERROR;
-  }
-
 could_not_set_format:
   {
     GST_ELEMENT_ERROR (sink, LIBRARY, SETTINGS, (NULL),
         ("Error setting connection format: %s", shout_get_error (sink->conn)));
-    return GST_FLOW_ERROR;
+    return FALSE;
   }
 
 could_not_connect:
@@ -707,16 +550,7 @@ could_not_connect:
         ("shout_open() failed: err=%s", shout_get_error (sink->conn)));
     g_signal_emit (sink, gst_shout2send_signals[SIGNAL_CONNECTION_PROBLEM], 0,
         shout_get_errno (sink->conn));
-    return GST_FLOW_ERROR;
-  }
-
-connection_timeout:
-  {
-    GST_ELEMENT_ERROR (sink, RESOURCE, OPEN_WRITE,
-        (_("Could not connect to server")), ("connection timed out"));
-    g_signal_emit (sink, gst_shout2send_signals[SIGNAL_CONNECTION_PROBLEM], 0,
-        shout_get_errno (sink->conn));
-    return GST_FLOW_ERROR;
+    return FALSE;
   }
 }
 
@@ -738,8 +572,6 @@ gst_shout2send_stop (GstBaseSink * basesink)
   }
 
   sink->connected = FALSE;
-  sink->format = -1;
-  sink->usage = SHOUT_USAGE_UNKNOWN;
 
   return TRUE;
 }
@@ -776,20 +608,16 @@ gst_shout2send_render (GstBaseSink * basesink, GstBuffer * buf)
   GstShout2send *sink;
   glong ret;
   gint delay;
-  GstFlowReturn fret = GST_FLOW_OK;
+  GstFlowReturn fret;
   GstMapInfo map;
-  GstClockTime now;
-  ssize_t queuelen;
 
   sink = GST_SHOUT2SEND (basesink);
 
-  /* we connect here because we need to know the format before we can set up
-   * the connection, which we don't know yet in _start(), and also because we
-   * don't want to block the application thread */
+  /* presumably we connect here because we need to know the format before
+   * we can set up the connection, which we don't know yet in _start() */
   if (!sink->connected) {
-    fret = gst_shout2send_connect (sink);
-    if (fret != GST_FLOW_OK)
-      goto done;
+    if (!gst_shout2send_connect (sink))
+      return GST_FLOW_ERROR;
   }
 
   delay = shout_delay (sink->conn);
@@ -801,63 +629,20 @@ gst_shout2send_render (GstBaseSink * basesink, GstBuffer * buf)
 
       fret = gst_base_sink_wait_preroll (basesink);
       if (fret != GST_FLOW_OK)
-        goto done;
+        return fret;
     }
   } else {
     GST_LOG_OBJECT (sink, "we're %d msec late", -delay);
   }
 
-  /* accumulate how much data have actually been sent
-   * to the network since the last call to shout_send() */
-  queuelen = shout_queuelen (sink->conn);
-  if (sink->prev_queuelen > 0)
-    sink->data_sent += sink->prev_queuelen - queuelen;
-
   gst_buffer_map (buf, &map, GST_MAP_READ);
-
-  /* add map.size instead of re-reading the queue length because
-   * the data may actually be sent immediately */
-  sink->prev_queuelen = queuelen + map.size;
-
-  GST_LOG_OBJECT (sink, "sending %u bytes of data, queue length now is %"
-      G_GUINT64_FORMAT, (guint) map.size, sink->prev_queuelen);
-
+  GST_LOG_OBJECT (sink, "sending %u bytes of data", (guint) map.size);
   ret = shout_send (sink->conn, map.data, map.size);
-
   gst_buffer_unmap (buf, &map);
   if (ret != SHOUTERR_SUCCESS)
     goto send_error;
 
-  now = gst_util_get_timestamp ();
-  if (now - sink->datasent_reset_ts >= 500 * GST_MSECOND) {
-    guint64 send_rate;
-
-    send_rate = gst_util_uint64_scale (sink->data_sent, GST_SECOND,
-        now - sink->datasent_reset_ts);
-
-    if (send_rate == 0 && !sink->stalled) {
-      sink->stalled = TRUE;
-      sink->stalled_ts = now;
-    } else if (send_rate > 0 && sink->stalled) {
-      sink->stalled = FALSE;
-    }
-
-    sink->data_sent = 0;
-    sink->datasent_reset_ts = now;
-
-    GST_DEBUG_OBJECT (sink, "sending rate is %" G_GUINT64_FORMAT " bps, "
-        "stalled %d, stalled_ts %" GST_TIME_FORMAT, send_rate, sink->stalled,
-        GST_TIME_ARGS (sink->stalled_ts));
-
-    if (sink->stalled && now - sink->stalled_ts >= sink->timeout * GST_MSECOND) {
-      GST_WARNING_OBJECT (sink, "network send queue is stalled for too long");
-      goto network_error;
-    }
-  }
-
-done:
-
-  return fret;
+  return GST_FLOW_OK;
 
 /* ERRORS */
 send_error:
@@ -868,15 +653,6 @@ send_error:
         shout_get_errno (sink->conn));
     return GST_FLOW_ERROR;
   }
-
-network_error:
-  {
-    GST_ELEMENT_ERROR (sink, RESOURCE, WRITE, (NULL),
-        ("network timeout reached"));
-    g_signal_emit (sink, gst_shout2send_signals[SIGNAL_CONNECTION_PROBLEM], 0,
-        SHOUTERR_BUSY);
-    return GST_FLOW_ERROR;
-  }
 }
 
 static void
@@ -889,55 +665,53 @@ gst_shout2send_set_property (GObject * object, guint prop_id,
   switch (prop_id) {
 
     case ARG_IP:
-      g_free (shout2send->ip);
-      shout2send->ip = g_value_dup_string (value);
+      if (shout2send->ip)
+        g_free (shout2send->ip);
+      shout2send->ip = g_strdup (g_value_get_string (value));
       break;
     case ARG_PORT:
       shout2send->port = g_value_get_int (value);
       break;
     case ARG_PASSWORD:
-      g_free (shout2send->password);
-      shout2send->password = g_value_dup_string (value);
+      if (shout2send->password)
+        g_free (shout2send->password);
+      shout2send->password = g_strdup (g_value_get_string (value));
       break;
     case ARG_USERNAME:
-      g_free (shout2send->username);
-      shout2send->username = g_value_dup_string (value);
+      if (shout2send->username)
+        g_free (shout2send->username);
+      shout2send->username = g_strdup (g_value_get_string (value));
       break;
     case ARG_PUBLIC:
       shout2send->ispublic = g_value_get_boolean (value);
       break;
     case ARG_STREAMNAME:       /* Name of the stream */
-      g_free (shout2send->streamname);
-      shout2send->streamname = g_value_dup_string (value);
+      if (shout2send->streamname)
+        g_free (shout2send->streamname);
+      shout2send->streamname = g_strdup (g_value_get_string (value));
       break;
     case ARG_DESCRIPTION:      /* Description of the stream */
-      g_free (shout2send->description);
-      shout2send->description = g_value_dup_string (value);
+      if (shout2send->description)
+        g_free (shout2send->description);
+      shout2send->description = g_strdup (g_value_get_string (value));
       break;
     case ARG_GENRE:            /* Genre of the stream */
-      g_free (shout2send->genre);
-      shout2send->genre = g_value_dup_string (value);
+      if (shout2send->genre)
+        g_free (shout2send->genre);
+      shout2send->genre = g_strdup (g_value_get_string (value));
       break;
     case ARG_PROTOCOL:         /* protocol to connect with */
       shout2send->protocol = g_value_get_enum (value);
       break;
     case ARG_MOUNT:            /* mountpoint of stream (icecast only) */
-      g_free (shout2send->mount);
-      shout2send->mount = g_value_dup_string (value);
+      if (shout2send->mount)
+        g_free (shout2send->mount);
+      shout2send->mount = g_strdup (g_value_get_string (value));
       break;
-    case ARG_URL:              /* the stream's homepage URL */
-      g_free (shout2send->url);
-      shout2send->url = g_value_dup_string (value);
-      break;
-    case ARG_TIMEOUT:
-      shout2send->timeout = g_value_get_uint (value);
-      break;
-    case ARG_SEND_TITLE_INFO:
-      shout2send->send_title_info = g_value_get_boolean (value);
-      break;
-    case ARG_USERAGENT:
-      g_free (shout2send->user_agent);
-      shout2send->user_agent = g_value_dup_string (value);
+    case ARG_URL:              /* Url of the stream (I'm guessing) */
+      if (shout2send->url)
+        g_free (shout2send->url);
+      shout2send->url = g_strdup (g_value_get_string (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -984,18 +758,9 @@ gst_shout2send_get_property (GObject * object, guint prop_id,
     case ARG_MOUNT:            /* mountpoint of stream (icecast only) */
       g_value_set_string (value, shout2send->mount);
       break;
-    case ARG_URL:              /* the stream's homepage URL */
+    case ARG_URL:              /* Url of stream (I'm guessing) */
       g_value_set_string (value, shout2send->url);
       break;
-    case ARG_TIMEOUT:
-      g_value_set_uint (value, shout2send->timeout);
-      break;
-    case ARG_SEND_TITLE_INFO:
-      g_value_set_boolean (value, shout2send->send_title_info);
-      break;
-    case ARG_USERAGENT:
-      g_value_set_string (value, shout2send->user_agent);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1016,24 +781,13 @@ gst_shout2send_setcaps (GstBaseSink * basesink, GstCaps * caps)
   GST_DEBUG_OBJECT (shout2send, "mimetype of caps given is: %s", mimetype);
 
   if (!strcmp (mimetype, "audio/mpeg")) {
-    shout2send->format = SHOUT_FORMAT_MP3;
-    shout2send->usage = SHOUT_USAGE_AUDIO;
-  } else if (g_str_has_suffix (mimetype, "/ogg")) {
-    shout2send->format = SHOUT_FORMAT_OGG;
-    if (g_str_has_prefix (mimetype, "audio/"))
-      shout2send->usage = SHOUT_USAGE_AUDIO;
-    else if (g_str_has_prefix (mimetype, "video/"))
-      shout2send->usage = SHOUT_USAGE_VISUAL | SHOUT_USAGE_AUDIO;
-    else
-      shout2send->usage = SHOUT_USAGE_UNKNOWN;
-  } else if (g_str_has_suffix (mimetype, "/webm")) {
-    shout2send->format = SHOUT_FORMAT_WEBM;
-    if (g_str_has_prefix (mimetype, "audio/"))
-      shout2send->usage = SHOUT_USAGE_AUDIO;
-    else if (g_str_has_prefix (mimetype, "video/"))
-      shout2send->usage = SHOUT_USAGE_VISUAL | SHOUT_USAGE_AUDIO;
-    else
-      shout2send->usage = SHOUT_USAGE_UNKNOWN;
+    shout2send->audio_format = SHOUT_FORMAT_MP3;
+  } else if (!strcmp (mimetype, "application/ogg")) {
+    shout2send->audio_format = SHOUT_FORMAT_VORBIS;
+#ifdef SHOUT_FORMAT_WEBM
+  } else if (!strcmp (mimetype, "video/webm")) {
+    shout2send->audio_format = SHOUT_FORMAT_WEBM;
+#endif
   } else {
     ret = FALSE;
   }
@@ -1044,11 +798,19 @@ gst_shout2send_setcaps (GstBaseSink * basesink, GstCaps * caps)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  return GST_ELEMENT_REGISTER (shout2send, plugin);
+#ifdef ENABLE_NLS
+  setlocale (LC_ALL, "");
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+#endif /* ENABLE_NLS */
+
+  return gst_element_register (plugin, "shout2send", GST_RANK_NONE,
+      GST_TYPE_SHOUT2SEND);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    shout2,
+    shout2send,
     "Sends data to an icecast server using libshout2",
-    plugin_init, VERSION, GST_LICENSE, GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
+    plugin_init,
+    VERSION, "LGPL", "libshout2", "http://www.icecast.org/download.html")
