@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -36,9 +36,13 @@ GST_DEBUG_CATEGORY_EXTERN (wavpack_debug);
 gboolean
 gst_wavpack_read_header (WavpackHeader * header, guint8 * buf)
 {
-  memmove (header, buf, sizeof (WavpackHeader));
+  g_memmove (header, buf, sizeof (WavpackHeader));
 
+#ifndef WAVPACK_OLD_API
   WavpackLittleEndianToNative (header, (char *) WavpackHeaderFormat);
+#else
+  little_endian_to_native (header, WavpackHeaderFormat);
+#endif
 
   return (memcmp (header->ckID, "wvpk", 4) == 0);
 }
@@ -108,24 +112,18 @@ gst_wavpack_get_default_channel_mask (gint nchannels)
     case 11:
       channel_mask |= 0x00400;
       channel_mask |= 0x00200;
-      /* FALLTHROUGH */
     case 9:
       channel_mask |= 0x00100;
-      /* FALLTHROUGH */
     case 8:
       channel_mask |= 0x00080;
       channel_mask |= 0x00040;
-      /* FALLTHROUGH */
     case 6:
       channel_mask |= 0x00020;
       channel_mask |= 0x00010;
-      /* FALLTHROUGH */
     case 4:
       channel_mask |= 0x00008;
-      /* FALLTHROUGH */
     case 3:
       channel_mask |= 0x00004;
-      /* FALLTHROUGH */
     case 2:
       channel_mask |= 0x00002;
       channel_mask |= 0x00001;
@@ -145,24 +143,24 @@ static const struct
   const GstAudioChannelPosition gst_pos;
 } layout_mapping[] = {
   {
-      0x00001, GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT}, {
-      0x00002, GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT}, {
-      0x00004, GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER}, {
-      0x00008, GST_AUDIO_CHANNEL_POSITION_LFE1}, {
-      0x00010, GST_AUDIO_CHANNEL_POSITION_REAR_LEFT}, {
-      0x00020, GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT}, {
-      0x00040, GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT_OF_CENTER}, {
-      0x00080, GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER}, {
-      0x00100, GST_AUDIO_CHANNEL_POSITION_REAR_CENTER}, {
-      0x00200, GST_AUDIO_CHANNEL_POSITION_SIDE_LEFT}, {
-      0x00400, GST_AUDIO_CHANNEL_POSITION_SIDE_RIGHT}, {
-      0x00800, GST_AUDIO_CHANNEL_POSITION_TOP_CENTER}, {
-      0x01000, GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_LEFT}, {
-      0x02000, GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_CENTER}, {
-      0x04000, GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_RIGHT}, {
-      0x08000, GST_AUDIO_CHANNEL_POSITION_TOP_REAR_LEFT}, {
-      0x10000, GST_AUDIO_CHANNEL_POSITION_TOP_REAR_CENTER}, {
-      0x20000, GST_AUDIO_CHANNEL_POSITION_TOP_REAR_RIGHT}
+  0x00001, GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT}, {
+  0x00002, GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT}, {
+  0x00004, GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER}, {
+  0x00008, GST_AUDIO_CHANNEL_POSITION_LFE1}, {
+  0x00010, GST_AUDIO_CHANNEL_POSITION_REAR_LEFT}, {
+  0x00020, GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT}, {
+  0x00040, GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT_OF_CENTER}, {
+  0x00080, GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER}, {
+  0x00100, GST_AUDIO_CHANNEL_POSITION_REAR_CENTER}, {
+  0x00200, GST_AUDIO_CHANNEL_POSITION_SIDE_LEFT}, {
+  0x00400, GST_AUDIO_CHANNEL_POSITION_SIDE_RIGHT}, {
+  0x00800, GST_AUDIO_CHANNEL_POSITION_TOP_CENTER}, {
+  0x01000, GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_LEFT}, {
+  0x02000, GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_CENTER}, {
+  0x04000, GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_RIGHT}, {
+  0x08000, GST_AUDIO_CHANNEL_POSITION_TOP_REAR_LEFT}, {
+  0x10000, GST_AUDIO_CHANNEL_POSITION_TOP_REAR_CENTER}, {
+  0x20000, GST_AUDIO_CHANNEL_POSITION_TOP_REAR_RIGHT}
 };
 
 #define MAX_CHANNEL_POSITIONS G_N_ELEMENTS (layout_mapping)
