@@ -1,5 +1,5 @@
 /* GStreamer Adaptive Multi-Rate parser plugin
- * Copyright (C) 2006 Edgard Lima <edgard.lima@gmail.com>
+ * Copyright (C) 2006 Edgard Lima <edgard.lima@indt.org.br>
  * Copyright (C) 2008 Nokia Corporation. All rights reserved.
  *
  * Contact: Stefan Kost <stefan.kost@nokia.com>
@@ -16,24 +16,24 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /**
  * SECTION:element-amrparse
- * @title: amrparse
  * @short_description: AMR parser
  * @see_also: #GstAmrnbDec, #GstAmrnbEnc
  *
  * This is an AMR parser capable of handling both narrow-band and wideband
  * formats.
  *
- * ## Example launch line
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
  * gst-launch-1.0 filesrc location=abc.amr ! amrparse ! amrdec ! audioresample ! audioconvert ! alsasink
  * ]|
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,9 +42,8 @@
 
 #include <string.h>
 
-#include "gstaudioparserselements.h"
 #include "gstamrparse.h"
-#include <gst/pbutils/pbutils.h>
+
 
 static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -82,12 +81,8 @@ static GstCaps *gst_amr_parse_sink_getcaps (GstBaseParse * parse,
 
 static GstFlowReturn gst_amr_parse_handle_frame (GstBaseParse * parse,
     GstBaseParseFrame * frame, gint * skipsize);
-static GstFlowReturn gst_amr_parse_pre_push_frame (GstBaseParse * parse,
-    GstBaseParseFrame * frame);
 
 G_DEFINE_TYPE (GstAmrParse, gst_amr_parse, GST_TYPE_BASE_PARSE);
-GST_ELEMENT_REGISTER_DEFINE (amrparse, "amrparse",
-    GST_RANK_PRIMARY + 1, GST_TYPE_AMR_PARSE);
 
 /**
  * gst_amr_parse_class_init:
@@ -103,8 +98,10 @@ gst_amr_parse_class_init (GstAmrParseClass * klass)
   GST_DEBUG_CATEGORY_INIT (amrparse_debug, "amrparse", 0,
       "AMR-NB audio stream parser");
 
-  gst_element_class_add_static_pad_template (element_class, &sink_template);
-  gst_element_class_add_static_pad_template (element_class, &src_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
 
   gst_element_class_set_static_metadata (element_class,
       "AMR audio stream parser", "Codec/Parser/Audio",
@@ -116,8 +113,6 @@ gst_amr_parse_class_init (GstAmrParseClass * klass)
   parse_class->set_sink_caps = GST_DEBUG_FUNCPTR (gst_amr_parse_sink_setcaps);
   parse_class->get_sink_caps = GST_DEBUG_FUNCPTR (gst_amr_parse_sink_getcaps);
   parse_class->handle_frame = GST_DEBUG_FUNCPTR (gst_amr_parse_handle_frame);
-  parse_class->pre_push_frame =
-      GST_DEBUG_FUNCPTR (gst_amr_parse_pre_push_frame);
 }
 
 
@@ -133,8 +128,7 @@ gst_amr_parse_init (GstAmrParse * amrparse)
   /* init rest */
   gst_base_parse_set_min_frame_size (GST_BASE_PARSE (amrparse), 62);
   GST_DEBUG ("initialized");
-  GST_PAD_SET_ACCEPT_INTERSECT (GST_BASE_PARSE_SINK_PAD (amrparse));
-  GST_PAD_SET_ACCEPT_TEMPLATE (GST_BASE_PARSE_SINK_PAD (amrparse));
+
 }
 
 
@@ -263,6 +257,7 @@ gst_amr_parse_handle_frame (GstBaseParse * parse,
   GstMapInfo map;
   gint fsize = 0, mode, dsize;
   GstAmrParse *amrparse;
+  GstFlowReturn ret = GST_FLOW_OK;
   gboolean found = FALSE;
 
   amrparse = GST_AMR_PARSE (parse);
@@ -312,21 +307,18 @@ gst_amr_parse_handle_frame (GstBaseParse * parse,
       } else if (GST_BASE_PARSE_DRAINING (parse)) {
         /* not enough, but draining, so ok */
         found = TRUE;
-        fsize = map.size;
       }
     }
   }
 
-  found = found && fsize <= map.size;
-
 done:
   gst_buffer_unmap (buffer, &map);
 
-  if (found) {
-    return gst_base_parse_finish_frame (parse, frame, fsize);
+  if (found && fsize <= map.size) {
+    ret = gst_base_parse_finish_frame (parse, frame, fsize);
   }
 
-  return GST_FLOW_OK;
+  return ret;
 }
 
 /**
@@ -346,7 +338,6 @@ gst_amr_parse_start (GstBaseParse * parse)
   GST_DEBUG ("start");
   amrparse->need_header = TRUE;
   amrparse->header = 0;
-  amrparse->sent_codec_tag = FALSE;
   return TRUE;
 }
 
@@ -379,7 +370,7 @@ gst_amr_parse_sink_getcaps (GstBaseParse * parse, GstCaps * filter)
 
 
   templ = gst_pad_get_pad_template_caps (GST_BASE_PARSE_SINK_PAD (parse));
-  peercaps = gst_pad_peer_query_caps (GST_BASE_PARSE_SRC_PAD (parse), filter);
+  peercaps = gst_pad_get_allowed_caps (GST_BASE_PARSE_SRC_PAD (parse));
 
   if (peercaps) {
     guint i, n;
@@ -398,7 +389,6 @@ gst_amr_parse_sink_getcaps (GstBaseParse * parse, GstCaps * filter)
 
     res = gst_caps_intersect_full (peercaps, templ, GST_CAPS_INTERSECT_FIRST);
     gst_caps_unref (peercaps);
-    res = gst_caps_make_writable (res);
     /* Append the template caps because we still want to accept
      * caps without any fields in the case upstream does not
      * know anything.
@@ -419,41 +409,3 @@ gst_amr_parse_sink_getcaps (GstBaseParse * parse, GstCaps * filter)
 
   return res;
 }
-
-static GstFlowReturn
-gst_amr_parse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
-{
-  GstAmrParse *amrparse = GST_AMR_PARSE (parse);
-
-  if (!amrparse->sent_codec_tag) {
-    GstTagList *taglist;
-    GstCaps *caps;
-
-    /* codec tag */
-    caps = gst_pad_get_current_caps (GST_BASE_PARSE_SRC_PAD (parse));
-    if (G_UNLIKELY (caps == NULL)) {
-      if (GST_PAD_IS_FLUSHING (GST_BASE_PARSE_SRC_PAD (parse))) {
-        GST_INFO_OBJECT (parse, "Src pad is flushing");
-        return GST_FLOW_FLUSHING;
-      } else {
-        GST_INFO_OBJECT (parse, "Src pad is not negotiated!");
-        return GST_FLOW_NOT_NEGOTIATED;
-      }
-    }
-
-    taglist = gst_tag_list_new_empty ();
-    gst_pb_utils_add_codec_description_to_tag_list (taglist,
-        GST_TAG_AUDIO_CODEC, caps);
-    gst_caps_unref (caps);
-
-    gst_base_parse_merge_tags (parse, taglist, GST_TAG_MERGE_REPLACE);
-    gst_tag_list_unref (taglist);
-
-    /* also signals the end of first-frame processing */
-    amrparse->sent_codec_tag = TRUE;
-  }
-
-  frame->flags |= GST_BASE_PARSE_FRAME_FLAG_CLIP;
-
-  return GST_FLOW_OK;
-}
