@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 /**
  * SECTION:element-mpegaudioparse
@@ -29,8 +29,7 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch-1.0 filesrc location=test.mp3 ! mpegaudioparse ! mpg123audiodec
- *  ! audioconvert ! audioresample ! autoaudiosink
+ * gst-launch-1.0 filesrc location=test.mp3 ! mpegaudioparse ! mad ! autoaudiosink
  * ]|
  * </refsect2>
  */
@@ -50,7 +49,6 @@
 
 #include "gstmpegaudioparse.h"
 #include <gst/base/gstbytereader.h>
-#include <gst/pbutils/pbutils.h>
 
 GST_DEBUG_CATEGORY_STATIC (mpeg_audio_parse_debug);
 #define GST_CAT_DEFAULT mpeg_audio_parse_debug
@@ -178,8 +176,10 @@ gst_mpeg_audio_parse_class_init (GstMpegAudioParseClass * klass)
 
   g_type_class_ref (GST_TYPE_MPEG_AUDIO_CHANNEL_MODE);
 
-  gst_element_class_add_static_pad_template (element_class, &sink_template);
-  gst_element_class_add_static_pad_template (element_class, &src_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
 
   gst_element_class_set_static_metadata (element_class, "MPEG1 Audio Parser",
       "Codec/Parser/Audio",
@@ -206,9 +206,8 @@ gst_mpeg_audio_parse_reset (GstMpegAudioParse * mp3parse)
   mp3parse->xing_total_time = 0;
   mp3parse->xing_bytes = 0;
   mp3parse->xing_vbr_scale = 0;
-  memset (mp3parse->xing_seek_table, 0, sizeof (mp3parse->xing_seek_table));
-  memset (mp3parse->xing_seek_table_inverse, 0,
-      sizeof (mp3parse->xing_seek_table_inverse));
+  memset (mp3parse->xing_seek_table, 0, 100);
+  memset (mp3parse->xing_seek_table_inverse, 0, 256);
 
   mp3parse->vbri_bitrate = 0;
   mp3parse->vbri_frames = 0;
@@ -226,8 +225,6 @@ static void
 gst_mpeg_audio_parse_init (GstMpegAudioParse * mp3parse)
 {
   gst_mpeg_audio_parse_reset (mp3parse);
-  GST_PAD_SET_ACCEPT_INTERSECT (GST_BASE_PARSE_SINK_PAD (mp3parse));
-  GST_PAD_SET_ACCEPT_TEMPLATE (GST_BASE_PARSE_SINK_PAD (mp3parse));
 }
 
 static void
@@ -435,7 +432,7 @@ gst_mp3parse_validate_extended (GstMpegAudioParse * mp3parse, GstBuffer * buf,
     if (G_UNLIKELY (!bpf)) {
       GST_DEBUG_OBJECT (mp3parse, "next header invalid (bitrate 0)");
       *valid = FALSE;
-      goto cleanup;
+      return TRUE;
     }
 
     offset += bpf;
@@ -607,7 +604,7 @@ gst_mpeg_audio_parse_handle_frame (GstBaseParse * parse,
   GstMpegAudioParse *mp3parse = GST_MPEG_AUDIO_PARSE (parse);
   GstBuffer *buf = frame->buffer;
   GstByteReader reader;
-  gint off, bpf = 0;
+  gint off, bpf;
   gboolean lost_sync, draining, valid, caps_change;
   guint32 header;
   guint bitrate, layer, rate, channels, version, mode, crc;
@@ -673,7 +670,7 @@ gst_mpeg_audio_parse_handle_frame (GstBaseParse * parse,
         /* not enough data */
         gst_base_parse_set_min_frame_size (parse, valid);
         *skipsize = 0;
-        goto cleanup;
+        return FALSE;
       } else {
         GST_DEBUG_OBJECT (parse, "determined freeform size %d", valid);
         mp3parse->freerate = valid;
@@ -685,7 +682,7 @@ gst_mpeg_audio_parse_handle_frame (GstBaseParse * parse,
     if (!bpf) {
       /* did not come up with valid freeform length, reject after all */
       *skipsize = 1;
-      goto cleanup;
+      return FALSE;
     }
   }
 
@@ -928,7 +925,7 @@ gst_mpeg_audio_parse_handle_first_frame (GstMpegAudioParse * mp3parse,
 
         if (table[percent] == i) {
           mp3parse->xing_seek_table_inverse[i] = percent * 100;
-        } else if (percent < 99 && table[percent]) {
+        } else if (table[percent] < i && percent < 99) {
           gdouble fa, fb, fx;
           gint a = percent, b = percent + 1;
 
@@ -949,9 +946,8 @@ gst_mpeg_audio_parse_handle_first_frame (GstMpegAudioParse * mp3parse,
     skip_toc:
       data += 100;
     } else {
-      memset (mp3parse->xing_seek_table, 0, sizeof (mp3parse->xing_seek_table));
-      memset (mp3parse->xing_seek_table_inverse, 0,
-          sizeof (mp3parse->xing_seek_table_inverse));
+      memset (mp3parse->xing_seek_table, 0, 100);
+      memset (mp3parse->xing_seek_table_inverse, 0, 256);
     }
 
     if (xing_flags & XING_VBR_SCALE_FLAG) {
@@ -1090,6 +1086,13 @@ gst_mpeg_audio_parse_handle_first_frame (GstMpegAudioParse * mp3parse,
         goto out_vbri;
       }
 
+      if (avail < offset_vbri + 26) {
+        GST_DEBUG_OBJECT (mp3parse,
+            "Not enough data to read VBRI header (need %d)",
+            offset_vbri + 26 + nseek_points * seek_bytes);
+        goto cleanup;
+      }
+
       data = map.data;
       data += offset_vbri + 26;
 
@@ -1322,16 +1325,47 @@ gst_mpeg_audio_parse_pre_push_frame (GstBaseParse * parse,
     GstBaseParseFrame * frame)
 {
   GstMpegAudioParse *mp3parse = GST_MPEG_AUDIO_PARSE (parse);
-  GstTagList *taglist = NULL;
+  GstTagList *taglist;
+
+  /* tag sending done late enough in hook to ensure pending events
+   * have already been sent */
+
+  if (!mp3parse->sent_codec_tag) {
+    gchar *codec;
+
+    /* codec tag */
+    if (mp3parse->layer == 3) {
+      codec = g_strdup_printf ("MPEG %d Audio, Layer %d (MP3)",
+          mp3parse->version, mp3parse->layer);
+    } else {
+      codec = g_strdup_printf ("MPEG %d Audio, Layer %d",
+          mp3parse->version, mp3parse->layer);
+    }
+    taglist = gst_tag_list_new (GST_TAG_AUDIO_CODEC, codec, NULL);
+    if (mp3parse->hdr_bitrate > 0 && mp3parse->xing_bitrate == 0 &&
+        mp3parse->vbri_bitrate == 0) {
+      /* We don't have a VBR bitrate, so post the available bitrate as
+       * nominal and let baseparse calculate the real bitrate */
+      gst_tag_list_add (taglist, GST_TAG_MERGE_REPLACE,
+          GST_TAG_NOMINAL_BITRATE, mp3parse->hdr_bitrate, NULL);
+    }
+    gst_pad_push_event (GST_BASE_PARSE_SRC_PAD (mp3parse),
+        gst_event_new_tag (taglist));
+    g_free (codec);
+
+    /* also signals the end of first-frame processing */
+    mp3parse->sent_codec_tag = TRUE;
+  }
 
   /* we will create a taglist (if any of the parameters has changed)
    * to add the tags that changed */
+  taglist = NULL;
   if (mp3parse->last_posted_crc != mp3parse->last_crc) {
     gboolean using_crc;
 
-    if (!taglist)
+    if (!taglist) {
       taglist = gst_tag_list_new_empty ();
-
+    }
     mp3parse->last_posted_crc = mp3parse->last_crc;
     if (mp3parse->last_posted_crc == CRC_PROTECTED) {
       using_crc = TRUE;
@@ -1343,56 +1377,19 @@ gst_mpeg_audio_parse_pre_push_frame (GstBaseParse * parse,
   }
 
   if (mp3parse->last_posted_channel_mode != mp3parse->last_mode) {
-    if (!taglist)
+    if (!taglist) {
       taglist = gst_tag_list_new_empty ();
-
+    }
     mp3parse->last_posted_channel_mode = mp3parse->last_mode;
 
     gst_tag_list_add (taglist, GST_TAG_MERGE_REPLACE, GST_TAG_MODE,
         gst_mpeg_audio_channel_mode_get_nick (mp3parse->last_mode), NULL);
   }
 
-  /* tag sending done late enough in hook to ensure pending events
-   * have already been sent */
-  if (taglist != NULL || !mp3parse->sent_codec_tag) {
-    GstCaps *caps;
-
-    if (taglist == NULL)
-      taglist = gst_tag_list_new_empty ();
-
-    /* codec tag */
-    caps = gst_pad_get_current_caps (GST_BASE_PARSE_SRC_PAD (parse));
-    if (G_UNLIKELY (caps == NULL)) {
-      gst_tag_list_unref (taglist);
-
-      if (GST_PAD_IS_FLUSHING (GST_BASE_PARSE_SRC_PAD (parse))) {
-        GST_INFO_OBJECT (parse, "Src pad is flushing");
-        return GST_FLOW_FLUSHING;
-      } else {
-        GST_INFO_OBJECT (parse, "Src pad is not negotiated!");
-        return GST_FLOW_NOT_NEGOTIATED;
-      }
-    }
-    gst_pb_utils_add_codec_description_to_tag_list (taglist,
-        GST_TAG_AUDIO_CODEC, caps);
-    gst_caps_unref (caps);
-
-    if (mp3parse->hdr_bitrate > 0 && mp3parse->xing_bitrate == 0 &&
-        mp3parse->vbri_bitrate == 0) {
-      /* We don't have a VBR bitrate, so post the available bitrate as
-       * nominal and let baseparse calculate the real bitrate */
-      gst_tag_list_add (taglist, GST_TAG_MERGE_REPLACE,
-          GST_TAG_NOMINAL_BITRATE, mp3parse->hdr_bitrate, NULL);
-    }
-
-    /* also signals the end of first-frame processing */
-    mp3parse->sent_codec_tag = TRUE;
-  }
-
-  /* if the taglist exists, we need to update it so it gets sent out */
+  /* if the taglist exists, we need to send it */
   if (taglist) {
-    gst_base_parse_merge_tags (parse, taglist, GST_TAG_MERGE_REPLACE);
-    gst_tag_list_unref (taglist);
+    gst_pad_push_event (GST_BASE_PARSE_SRC_PAD (mp3parse),
+        gst_event_new_tag (taglist));
   }
 
   /* usual clipping applies */
@@ -1401,19 +1398,6 @@ gst_mpeg_audio_parse_pre_push_frame (GstBaseParse * parse,
   return GST_FLOW_OK;
 }
 
-static void
-remove_fields (GstCaps * caps)
-{
-  guint i, n;
-
-  n = gst_caps_get_size (caps);
-  for (i = 0; i < n; i++) {
-    GstStructure *s = gst_caps_get_structure (caps, i);
-
-    gst_structure_remove_field (s, "parsed");
-  }
-}
-
 static GstCaps *
 gst_mpeg_audio_parse_get_sink_caps (GstBaseParse * parse, GstCaps * filter)
 {
@@ -1421,23 +1405,28 @@ gst_mpeg_audio_parse_get_sink_caps (GstBaseParse * parse, GstCaps * filter)
   GstCaps *res;
 
   templ = gst_pad_get_pad_template_caps (GST_BASE_PARSE_SINK_PAD (parse));
-  if (filter) {
-    GstCaps *fcopy = gst_caps_copy (filter);
-    /* Remove the fields we convert */
-    remove_fields (fcopy);
-    peercaps = gst_pad_peer_query_caps (GST_BASE_PARSE_SRC_PAD (parse), fcopy);
-    gst_caps_unref (fcopy);
-  } else
-    peercaps = gst_pad_peer_query_caps (GST_BASE_PARSE_SRC_PAD (parse), NULL);
+  peercaps = gst_pad_get_allowed_caps (GST_BASE_PARSE_SRC_PAD (parse));
 
   if (peercaps) {
+    guint i, n;
+
     /* Remove the parsed field */
     peercaps = gst_caps_make_writable (peercaps);
-    remove_fields (peercaps);
+    n = gst_caps_get_size (peercaps);
+    for (i = 0; i < n; i++) {
+      GstStructure *s = gst_caps_get_structure (peercaps, i);
+
+      gst_structure_remove_field (s, "parsed");
+    }
 
     res = gst_caps_intersect_full (peercaps, templ, GST_CAPS_INTERSECT_FIRST);
     gst_caps_unref (peercaps);
-    gst_caps_unref (templ);
+
+    /* Append the template caps because we still want to accept
+     * caps without any fields in the case upstream does not
+     * know anything.
+     */
+    gst_caps_append (res, templ);
   } else {
     res = templ;
   }
