@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 /*
  * Unless otherwise indicated, Source Code is licensed under MIT license.
@@ -45,11 +45,11 @@
 
 #include <glib.h>
 #include <string.h>
-#include <gst/video/video.h>
 
 #include "descriptors.h"
 #include "properties.h"
 #include "fourcc.h"
+#include "ftypcc.h"
 
 /* helper storage struct */
 #define ATOM_ARRAY(struct_type) \
@@ -103,10 +103,9 @@ typedef enum _AtomsTreeFlavor
 typedef struct _AtomsContext
 {
   AtomsTreeFlavor flavor;
-  gboolean force_create_timecode_trak;
 } AtomsContext;
 
-AtomsContext* atoms_context_new  (AtomsTreeFlavor flavor, gboolean force_create_timecode_trak);
+AtomsContext* atoms_context_new  (AtomsTreeFlavor flavor);
 void          atoms_context_free (AtomsContext *context);
 
 #define METADATA_DATA_FLAG 0x0
@@ -114,9 +113,7 @@ void          atoms_context_free (AtomsContext *context);
 
 /* atom defs and functions */
 
-typedef struct _AtomInfo AtomInfo;
-
-/*
+/**
  * Used for storing time related values for some atoms.
  */
 typedef struct _TimeInfo
@@ -253,8 +250,6 @@ typedef struct _AtomHDLR
   guint32 flags;
   guint32 flags_mask;
   gchar *name;
-
-  AtomsTreeFlavor flavor;
 } AtomHDLR;
 
 typedef struct _AtomVMHD
@@ -285,53 +280,6 @@ typedef struct _AtomHMHD
   guint32 sliding_avg_bitrate;
 } AtomHMHD;
 
-typedef struct _AtomTCMI
-{
-  AtomFull header;
-
-  guint16 text_font;
-  guint16 text_face;
-  guint16 text_size;
-  guint16 text_color[3];
-  guint16 bg_color[3];
-  gchar *font_name;
-} AtomTCMI;
-
-typedef struct _AtomTMCD
-{
-  Atom header;
-
-  AtomTCMI tcmi;
-} AtomTMCD;
-
-typedef struct _AtomGMIN
-{
-  AtomFull header;
-
-  guint16 graphics_mode;
-  guint16 opcolor[3];
-  guint8 balance;
-  guint8 reserved;
-
-} AtomGMIN;
-
-typedef struct _AtomGMHD
-{
-  Atom header;
-
-  /* Only gmin is required in a gmhd atom
-   * The other fields are optional */
-  AtomGMIN gmin;
-  AtomTMCD *tmcd;
-
-} AtomGMHD;
-
-typedef struct _AtomNMHD
-{
-  Atom header;
-  guint32 flags;
-} AtomNMHD;
-
 typedef struct _AtomURL
 {
   AtomFull header;
@@ -391,10 +339,7 @@ typedef enum _SampleEntryKind
 {
   UNKNOWN,
   AUDIO,
-  VIDEO,
-  SUBTITLE,
-  TIMECODE,
-  CLOSEDCAPTION
+  VIDEO
 } SampleEntryKind;
 
 typedef struct _SampleTableEntry
@@ -404,7 +349,7 @@ typedef struct _SampleTableEntry
   guint8 reserved[6];
   guint16 data_reference_index;
 
-  /* type of entry */
+  /* sort of entry */
   SampleEntryKind kind;
 } SampleTableEntry;
 
@@ -468,39 +413,12 @@ typedef struct _SampleTableEntryMP4A
   GList *extension_atoms;
 } SampleTableEntryMP4A;
 
-typedef struct _AtomNAME
-{
-  Atom header;
-
-  guint8 language_code;
-  gchar *name;
-} AtomNAME;
-
-typedef struct _SampleTableEntryTMCD
+typedef struct _SampleTableEntryMP4S
 {
   SampleTableEntry se;
 
-  guint32 tc_flags;
-  guint32 timescale;
-  guint32 frame_duration;
-  guint8 n_frames;
-
-  AtomNAME name;
-
-} SampleTableEntryTMCD;
-
-typedef struct _SampleTableEntryTX3G
-{
-  SampleTableEntry se;
-
-  guint32 display_flags;
-  guint64 default_text_box;
-  guint16 font_id;
-  guint8  font_face; /* bold=0x1, italic=0x2, underline=0x4 */
-  guint8  font_size; /* should always be 0.05 multiplied by the video track header height */
-  guint32 foreground_color_rgba;
-
-} SampleTableEntryTX3G;
+  AtomESDS es;
+} SampleTableEntryMP4S;
 
 typedef struct _AtomSTSD
 {
@@ -537,28 +455,15 @@ typedef struct _AtomSTSC
   ATOM_ARRAY (STSCEntry) entries;
 } AtomSTSC;
 
-/* FIXME: this can support multiple tracks */
-typedef struct _AtomTREF
-{
-  Atom header;
-
-  guint32 reftype;
-  ATOM_ARRAY (guint32) entries;
-} AtomTREF;
 
 /*
  * used for both STCO and CO64
- * The table will be written out as STCO automatically when
- * the offsets being written will fit in a 32-bit table,
- * otherwise it is written as CO64
+ * if used as STCO, entries should be truncated to use only 32bits
  */
 typedef struct _AtomSTCO64
 {
   AtomFull header;
-  /* Global offset to add to entries when serialising */
-  guint32 chunk_offset;
-  /* Maximum offset stored in the table */
-  guint64 max_offset;
+
   ATOM_ARRAY (guint64) entries;
 } AtomSTCO64;
 
@@ -577,14 +482,6 @@ typedef struct _AtomCTTS
   gboolean do_pts;
 } AtomCTTS;
 
-typedef struct _AtomSVMI
-{
-  AtomFull header;
-
-  guint8 stereoscopic_composition_type;
-  gboolean is_left_first;
-} AtomSVMI;
-
 typedef struct _AtomSTBL
 {
   Atom header;
@@ -596,8 +493,6 @@ typedef struct _AtomSTBL
   AtomSTSZ stsz;
   /* NULL if not present */
   AtomCTTS *ctts;
-  /* NULL if not present */
-  AtomSVMI *svmi;
 
   AtomSTCO64 stco64;
 } AtomSTBL;
@@ -610,8 +505,6 @@ typedef struct _AtomMINF
   AtomVMHD *vmhd;
   AtomSMHD *smhd;
   AtomHMHD *hmhd;
-  AtomGMHD *gmhd;
-  AtomNMHD *nmhd;
 
   AtomHDLR *hdlr;
   AtomDINF dinf;
@@ -689,8 +582,6 @@ typedef struct _AtomUDTA
   GList* entries;
   /* or list is further down */
   AtomMETA *meta;
-
-  AtomsContext *context;
 } AtomUDTA;
 
 enum TrFlags
@@ -710,17 +601,7 @@ enum TfFlags
   TF_DEFAULT_SAMPLE_DURATION  = 0x08,     /* default-sample-duration-present */
   TF_DEFAULT_SAMPLE_SIZE      = 0x010,    /* default-sample-size-present */
   TF_DEFAULT_SAMPLE_FLAGS     = 0x020,    /* default-sample-flags-present */
-  TF_DURATION_IS_EMPTY        = 0x010000, /* sample-composition-time-offsets-presents */
-  TF_DEFAULT_BASE_IS_MOOF     = 0x020000  /* default-base-is-moof */
-};
-
-/* Timecode flags */
-enum TcFlags
-{
-  TC_DROP_FRAME = 0x0001,   /* Drop-frame timecode */
-  TC_24H_MAX = 0x0002,      /* Whether the timecode wraps after 24 hours */
-  TC_NEGATIVE_OK = 0x0004,  /* Whether negative time values are OK */
-  TC_COUNTER = 0x0008       /* Whether the time value corresponds to a tape counter value */
+  TF_DURATION_IS_EMPTY        = 0x010000  /* sample-composition-time-offsets-presents */
 };
 
 typedef struct _AtomTRAK
@@ -728,19 +609,12 @@ typedef struct _AtomTRAK
   Atom header;
 
   AtomTKHD tkhd;
-  AtomInfo *tapt;
   AtomEDTS *edts;
   AtomMDIA mdia;
-  AtomUDTA udta;
-  AtomTREF *tref;
 
   /* some helper info for structural conformity checks */
   gboolean is_video;
   gboolean is_h264;
-  gboolean is_ac3;
-  gboolean is_eac3;
-
-  AtomsContext *context;
 } AtomTRAK;
 
 typedef struct _AtomTREX
@@ -791,13 +665,6 @@ typedef struct _AtomTFHD
   guint32 default_sample_flags;
 } AtomTFHD;
 
-typedef struct _AtomTFDT
-{
-  AtomFull header;
-
-  guint64 base_media_decode_time;
-} AtomTFDT;
-
 typedef struct _TRUNSampleEntry
 {
   guint32 sample_duration;
@@ -835,9 +702,7 @@ typedef struct _AtomTRAF
 
   AtomTFHD tfhd;
 
-  AtomTFDT tfdt;
-
-  /* list of AtomTRUN. */
+  /* list of AtomTRUN */
   GList *truns;
   /* list of AtomSDTP */
   GList *sdtps;
@@ -851,8 +716,6 @@ typedef struct _AtomMOOF
 
   /* list of AtomTRAF */
   GList *trafs;
-
-  guint64 traf_offset;
 } AtomMOOF;
 
 
@@ -868,10 +731,9 @@ typedef struct _AtomMOOV
 
   /* list of AtomTRAK */
   GList *traks;
-  AtomUDTA udta;
+  AtomUDTA *udta;
 
   gboolean fragmented;
-  guint32 chunks_offset;
 } AtomMOOV;
 
 typedef struct _AtomWAVE
@@ -925,29 +787,13 @@ typedef guint64 (*AtomFreeFunc) (Atom *atom);
  * All we need are the two functions (copying it to an array
  * for serialization and the memory releasing function).
  */
-struct _AtomInfo
+typedef struct _AtomInfo
 {
   Atom *atom;
   AtomCopyDataFunc copy_data_func;
   AtomFreeFunc free_func;
-};
+} AtomInfo;
 
-typedef struct _EAC3BitstreamInfo
-{
-  guint8 strmtyp;
-  guint8 substreamid;
-  guint16 frmsiz;
-  guint8 fscod;
-  guint8 numblkscod;
-  guint8 acmod;
-  guint8 lfeon;
-  guint8 bsid;
-  guint8 bsmod;
-  guint16 chanmap;
-
-} EAC3BitstreamInfo;
-
-guint64    atoms_get_current_qt_time   (void);
 
 guint64    atom_copy_data              (Atom *atom, guint8 **buffer,
                                         guint64 *size, guint64* offset);
@@ -962,18 +808,14 @@ AtomTRAK*  atom_trak_new               (AtomsContext *context);
 void       atom_trak_add_samples       (AtomTRAK * trak, guint32 nsamples, guint32 delta,
                                         guint32 size, guint64 chunk_offset, gboolean sync,
                                         gint64 pts_offset);
-void       atom_trak_set_elst_entry    (AtomTRAK * trak, gint index, guint32 duration,
+void       atom_trak_add_elst_entry    (AtomTRAK * trak, guint32 duration,
                                         guint32 media_time, guint32 rate);
-void       atom_trak_edts_clear        (AtomTRAK * trak);
 guint32    atom_trak_get_timescale     (AtomTRAK *trak);
 guint32    atom_trak_get_id            (AtomTRAK * trak);
-void       atom_trak_set_constant_size_samples (AtomTRAK * trak, guint32 sample_size);
 void       atom_stbl_add_samples       (AtomSTBL * stbl, guint32 nsamples,
                                         guint32 delta, guint32 size,
                                         guint64 chunk_offset, gboolean sync,
                                         gint64 pts_offset);
-void       atom_stsc_add_new_entry     (AtomSTSC * stsc,
-                                        guint32 first_chunk, guint32 nsamples, guint32 sample_description_index);
 
 AtomMOOV*  atom_moov_new               (AtomsContext *context);
 void       atom_moov_free              (AtomMOOV *moov);
@@ -981,16 +823,12 @@ guint64    atom_moov_copy_data         (AtomMOOV *atom, guint8 **buffer, guint64
 void       atom_moov_update_timescale  (AtomMOOV *moov, guint32 timescale);
 void       atom_moov_update_duration   (AtomMOOV *moov);
 void       atom_moov_set_fragmented    (AtomMOOV *moov, gboolean fragmented);
-void       atom_moov_chunks_set_offset (AtomMOOV *moov, guint32 offset);
+void       atom_moov_chunks_add_offset (AtomMOOV *moov, guint32 offset);
 void       atom_moov_add_trak          (AtomMOOV *moov, AtomTRAK *trak);
-guint      atom_moov_get_trak_count    (AtomMOOV *moov);
-guint64    atom_moov_calc_stco_to_co64_conversion_total_offset (AtomMOOV *moov, guint64 offset);
-
-guint      atom_framerate_to_timescale (gint fps_n, gint fps_d);
 
 guint64    atom_mvhd_copy_data         (AtomMVHD * atom, guint8 ** buffer,
                                         guint64 * size, guint64 * offset);
-void       atom_stco64_chunks_set_offset (AtomSTCO64 * stco64, guint32 offset);
+void       atom_stco64_chunks_add_offset (AtomSTCO64 * stco64, guint32 offset);
 guint64    atom_trak_copy_data         (AtomTRAK * atom, guint8 ** buffer,
                                         guint64 * size, guint64 * offset);
 void       atom_stbl_clear             (AtomSTBL * stbl);
@@ -1005,23 +843,17 @@ guint64    atom_stsz_copy_data         (AtomSTSZ *atom, guint8 **buffer,
                                         guint64 *size, guint64* offset);
 guint64    atom_ctts_copy_data         (AtomCTTS *atom, guint8 **buffer,
                                         guint64 *size, guint64* offset);
-guint64    atom_svmi_copy_data         (AtomSVMI *atom, guint8 **buffer,
-                                        guint64 *size, guint64* offset);
-AtomSVMI * atom_svmi_new (guint8 stereoscopic_composition_type, gboolean is_left_first);
 guint64    atom_stco64_copy_data       (AtomSTCO64 *atom, guint8 **buffer,
                                         guint64 *size, guint64* offset);
 AtomMOOF*  atom_moof_new               (AtomsContext *context, guint32 sequence_number);
 void       atom_moof_free              (AtomMOOF *moof);
 guint64    atom_moof_copy_data         (AtomMOOF *moof, guint8 **buffer, guint64 *size, guint64* offset);
-void       atom_moof_set_base_offset   (AtomMOOF * moof, guint64 offset);
 AtomTRAF * atom_traf_new               (AtomsContext * context, guint32 track_ID);
 void       atom_traf_free              (AtomTRAF * traf);
-void       atom_traf_set_base_decode_time (AtomTRAF * traf, guint64 base_decode_time);
-void       atom_traf_add_samples       (AtomTRAF * traf, guint32 nsamples, guint32 delta,
-                                        guint32 size, gint32 data_offset, gboolean sync,
-                                        gint64 pts_offset, gboolean sdtp_sync);
+void       atom_traf_add_samples       (AtomTRAF * traf, guint32 delta,
+                                        guint32 size, gboolean sync, gint64 pts_offset,
+                                        gboolean sdtp_sync);
 guint32    atom_traf_get_sample_num    (AtomTRAF * traf);
-void       atom_trun_set_offset        (AtomTRUN * trun, gint32 offset);
 void       atom_moof_add_traf          (AtomMOOF *moof, AtomTRAF *traf);
 
 AtomMFRA*  atom_mfra_new               (AtomsContext *context);
@@ -1034,6 +866,7 @@ guint64    atom_mfra_copy_data         (AtomMFRA *mfra, guint8 **buffer, guint64
 
 
 /* media sample description related helpers */
+
 typedef struct
 {
   guint16 version;
@@ -1065,71 +898,33 @@ typedef struct
   GstBuffer *codec_data;
 } AudioSampleEntry;
 
-typedef struct
-{
-  guint32 fourcc;
-
-  guint8  font_face; /* bold=0x1, italic=0x2, underline=0x4 */
-  guint8  font_size;
-  guint32 foreground_color_rgba;
-} SubtitleSampleEntry;
-
-void subtitle_sample_entry_init (SubtitleSampleEntry * entry);
-
-SampleTableEntryMP4A * atom_trak_set_audio_type (AtomTRAK * trak, AtomsContext * context,
+void atom_trak_set_audio_type (AtomTRAK * trak, AtomsContext * context,
                                AudioSampleEntry * entry, guint32 scale,
                                AtomInfo * ext, gint sample_size);
 
-SampleTableEntryMP4V * atom_trak_set_video_type (AtomTRAK * trak, AtomsContext * context,
+void atom_trak_set_video_type (AtomTRAK * trak, AtomsContext * context,
                                VisualSampleEntry * entry, guint32 rate,
                                GList * ext_atoms_list);
 
-SampleTableEntryTX3G * atom_trak_set_subtitle_type (AtomTRAK * trak, AtomsContext * context,
-                               SubtitleSampleEntry * entry);
-
-SampleTableEntryTMCD *
-atom_trak_set_timecode_type (AtomTRAK * trak, AtomsContext * context, guint trak_timescale, GstVideoTimeCode * tc);
-
-SampleTableEntry * atom_trak_set_caption_type (AtomTRAK *trak, AtomsContext *context,
-					       guint32 trak_timescale, guint32 caption_type);
-
 void atom_trak_update_bitrates (AtomTRAK * trak, guint32 avg_bitrate,
                                 guint32 max_bitrate);
 
-void atom_trak_tx3g_update_dimension (AtomTRAK * trak, guint32 width,
-                                      guint32 height);
-
-void sample_table_entry_add_ext_atom (SampleTableEntry * ste, AtomInfo * ext);
-
 AtomInfo *   build_codec_data_extension  (guint32 fourcc, const GstBuffer * codec_data);
 AtomInfo *   build_mov_aac_extension     (AtomTRAK * trak, const GstBuffer * codec_data,
                                           guint32 avg_bitrate, guint32 max_bitrate);
-AtomInfo *   build_mov_alac_extension    (const GstBuffer * codec_data);
+AtomInfo *   build_mov_alac_extension    (AtomTRAK * trak, const GstBuffer * codec_data);
 AtomInfo *   build_esds_extension        (AtomTRAK * trak, guint8 object_type,
                                           guint8 stream_type, const GstBuffer * codec_data,
                                           guint32 avg_bitrate, guint32 max_bitrate);
 AtomInfo *   build_btrt_extension        (guint32 buffer_size_db, guint32 avg_bitrate,
                                           guint32 max_bitrate);
-AtomInfo *   build_jp2h_extension        (gint width, gint height, const gchar *colorspace,
-                                          gint ncomp, const GValue * cmap_array,
+AtomInfo *   build_jp2h_extension        (AtomTRAK * trak, gint width, gint height,
+                                          const gchar *colorspace, gint ncomp,
+                                          const GValue * cmap_array,
                                           const GValue * cdef_array);
 
 AtomInfo *   build_jp2x_extension        (const GstBuffer * prefix);
-AtomInfo *   build_fiel_extension        (GstVideoInterlaceMode mode, GstVideoFieldOrder order);
-AtomInfo *   build_colr_extension        (const GstVideoColorimetry *colorimetry, gboolean is_mp4);
-AtomInfo *   build_clap_extension        (gint width_n, gint width_d, gint height_n, gint height_d, gint h_off_n, gint h_off_d, gint v_off_n, gint v_off_d);
-AtomInfo *   build_tapt_extension        (gint clef_width, gint clef_height, gint prof_width, gint prof_height, gint enof_width, gint enof_height);
-
-
-AtomInfo *   build_ac3_extension         (guint8 fscod, guint8 bsid,
-                                          guint8 bsmod, guint8 acmod,
-                                          guint8 lfe_on, guint8 bitrate_code);
-AtomInfo *   build_eac3_extension        (GArray *bitstreamInfo);
-AtomInfo *   build_opus_extension        (guint32 rate, guint8 channels, guint8 mapping_family,
-                                          guint8 stream_count, guint8 coupled_count,
-                                          guint8 channel_mapping[256], guint16 pre_skip,
-                                          guint16 output_gain);
-
+AtomInfo *   build_fiel_extension        (gint fields);
 AtomInfo *   build_amr_extension         (void);
 AtomInfo *   build_h263_extension        (void);
 AtomInfo *   build_gama_atom             (gdouble gamma);
@@ -1137,37 +932,28 @@ AtomInfo *   build_SMI_atom              (const GstBuffer *seqh);
 AtomInfo *   build_ima_adpcm_extension   (gint channels, gint rate,
                                           gint blocksize);
 AtomInfo *   build_uuid_xmp_atom         (GstBuffer * xmp);
-AtomInfo *   build_vpcC_extension        (guint8 profile, guint8 level, guint8 bit_depth,
-                                          guint8 chroma_subsampling, gboolean video_full_range,
-                                          guint8 colour_primaries, guint8 transfer_characteristics,
-                                          guint8 matrix_coefficients);
-AtomInfo *   build_vvcC_extension        (guint8 version, guint32 flags, const GstBuffer* codec_data);
 
 
 /*
  * Meta tags functions
  */
-void atom_udta_clear_tags (AtomUDTA *udta);
-void atom_udta_add_str_tag    (AtomUDTA *udta, guint32 fourcc, const gchar *value);
-void atom_udta_add_uint_tag   (AtomUDTA *udta, guint32 fourcc, guint32 flags,
+void atom_moov_add_str_tag    (AtomMOOV *moov, guint32 fourcc, const gchar *value);
+void atom_moov_add_uint_tag   (AtomMOOV *moov, guint32 fourcc, guint32 flags,
                                guint32 value);
-void atom_udta_add_tag        (AtomUDTA *udta, guint32 fourcc, guint32 flags,
+void atom_moov_add_tag        (AtomMOOV *moov, guint32 fourcc, guint32 flags,
                                const guint8 * data, guint size);
-void atom_udta_add_blob_tag   (AtomUDTA *udta, guint8 *data, guint size);
+void atom_moov_add_blob_tag   (AtomMOOV *moov, guint8 *data, guint size);
 
-void atom_udta_add_3gp_str_tag       (AtomUDTA *udta, guint32 fourcc, const gchar * value);
-void atom_udta_add_3gp_uint_tag      (AtomUDTA *udta, guint32 fourcc, guint16 value);
-void atom_udta_add_3gp_str_int_tag   (AtomUDTA *udta, guint32 fourcc, const gchar * value,
+void atom_moov_add_3gp_str_tag       (AtomMOOV * moov, guint32 fourcc, const gchar * value);
+void atom_moov_add_3gp_uint_tag      (AtomMOOV * moov, guint32 fourcc, guint16 value);
+void atom_moov_add_3gp_str_int_tag   (AtomMOOV * moov, guint32 fourcc, const gchar * value,
                                       gint16 ivalue);
-void atom_udta_add_3gp_tag           (AtomUDTA *udta, guint32 fourcc, guint8 * data,
+void atom_moov_add_3gp_tag           (AtomMOOV * moov, guint32 fourcc, guint8 * data,
                                       guint size);
 
-void atom_udta_add_xmp_tags          (AtomUDTA *udta, GstBuffer * xmp);
+void atom_moov_add_xmp_tags          (AtomMOOV * moov, GstBuffer * xmp);
 
-AtomTREF * atom_tref_new (guint32 reftype);
-void atom_tref_add_entry (AtomTREF * tref, guint32 sample);
-
-#define GST_QT_MUX_DEFAULT_TAG_LANGUAGE   "und" /* undefined/unknown */
+#define GST_QT_MUX_DEFAULT_TAG_LANGUAGE   "eng"
 guint16  language_code               (const char * lang);
 
 #endif /* __ATOMS_H__ */
