@@ -4,7 +4,6 @@
  *                    2006 Wim Taymans <wim@fluendo.com>
  *                    2006 David A. Schleef <ds@schleef.org>
  *                    2011 Collabora Ltd. <tim.muller@collabora.co.uk>
- *                    2015 Tim-Philipp Müller <tim@centricular.com>
  *
  * gstmultifilesink.c:
  *
@@ -20,47 +19,92 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 /**
  * SECTION:element-multifilesink
- * @title: multifilesink
  * @see_also: #GstFileSrc
  *
  * Write incoming data to a series of sequentially-named files.
  *
- * This element is usually used with data where each buffer is an
- * independent unit of data in its own right (e.g. raw video buffers or
- * encoded JPEG or PNG images) or with streamable container formats such
- * as MPEG-TS or MPEG-PS.
- *
- * It is not possible to use this element to create independently playable
- * mp4 files, use the splitmuxsink element for that instead.
- *
  * The filename property should contain a string with a \%d placeholder that will
  * be substituted with the index for each filename.
  *
- * If the #GstMultiFileSink:post-messages property is %TRUE, it sends an application
- * message named `GstMultiFileSink` after writing each buffer.
+ * If the #GstMultiFileSink:post-messages property is #TRUE, it sends an application
+ * message named
+ * <classname>&quot;GstMultiFileSink&quot;</classname> after writing each
+ * buffer.
  *
  * The message's structure contains these fields:
+ * <itemizedlist>
+ * <listitem>
+ *   <para>
+ *   #gchar *
+ *   <classname>&quot;filename&quot;</classname>:
+ *   the filename where the buffer was written.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #gint
+ *   <classname>&quot;index&quot;</classname>:
+ *   the index of the buffer.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
+ *   <classname>&quot;timestamp&quot;</classname>:
+ *   the timestamp of the buffer.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
+ *   <classname>&quot;stream-time&quot;</classname>:
+ *   the stream time of the buffer.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
+ *   <classname>&quot;running-time&quot;</classname>:
+ *   the running_time of the buffer.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
+ *   <classname>&quot;duration&quot;</classname>:
+ *   the duration of the buffer.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #guint64
+ *   <classname>&quot;offset&quot;</classname>:
+ *   the offset of the buffer that triggered the message.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #guint64
+ *   <classname>&quot;offset-end&quot;</classname>:
+ *   the offset-end of the buffer that triggered the message.
+ *   </para>
+ * </listitem>
+ * </itemizedlist>
  *
- * * #gchararray `filename`: the filename where the buffer was written.
- * * #gint `index`: index of the buffer.
- * * #GstClockTime `timestamp`: the timestamp of the buffer.
- * * #GstClockTime `stream-time`: the stream time of the buffer.
- * * #GstClockTime running-time`: the running_time of the buffer.
- * * #GstClockTime `duration`: the duration of the buffer.
- * * #guint64 `offset`: the offset of the buffer that triggered the message.
- * * #guint64 `offset-end`: the offset-end of the buffer that triggered the message.
- *
- * ## Example launch line
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
  * gst-launch-1.0 audiotestsrc ! multifilesink
- * gst-launch-1.0 videotestsrc ! multifilesink post-messages=true location="frame%d"
+ * gst-launch-1.0 videotestsrc ! multifilesink post-messages=true filename="frame%d"
  * ]|
+ * </refsect2>
  *
+ * Last reviewed on 2009-09-11 (0.10.17)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -85,8 +129,6 @@ GST_DEBUG_CATEGORY_STATIC (gst_multi_file_sink_debug);
 #define DEFAULT_NEXT_FILE GST_MULTI_FILE_SINK_NEXT_BUFFER
 #define DEFAULT_MAX_FILES 0
 #define DEFAULT_MAX_FILE_SIZE G_GUINT64_CONSTANT(2*1024*1024*1024)
-#define DEFAULT_MAX_FILE_DURATION GST_CLOCK_TIME_NONE
-#define DEFAULT_AGGREGATE_GOPS FALSE
 
 enum
 {
@@ -97,8 +139,7 @@ enum
   PROP_NEXT_FILE,
   PROP_MAX_FILES,
   PROP_MAX_FILE_SIZE,
-  PROP_MAX_FILE_DURATION,
-  PROP_AGGREGATE_GOPS
+  PROP_LAST
 };
 
 static void gst_multi_file_sink_finalize (GObject * object);
@@ -108,7 +149,6 @@ static void gst_multi_file_sink_set_property (GObject * object, guint prop_id,
 static void gst_multi_file_sink_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
-static gboolean gst_multi_file_sink_start (GstBaseSink * bsink);
 static gboolean gst_multi_file_sink_stop (GstBaseSink * sink);
 static GstFlowReturn gst_multi_file_sink_render (GstBaseSink * sink,
     GstBuffer * buffer);
@@ -120,8 +160,6 @@ static gboolean gst_multi_file_sink_open_next_file (GstMultiFileSink *
     multifilesink);
 static void gst_multi_file_sink_close_file (GstMultiFileSink * multifilesink,
     GstBuffer * buffer);
-static void gst_multi_file_sink_add_old_file (GstMultiFileSink * multifilesink,
-    gchar * fn);
 static void gst_multi_file_sink_ensure_max_files (GstMultiFileSink *
     multifilesink);
 static gboolean gst_multi_file_sink_event (GstBaseSink * sink,
@@ -131,7 +169,7 @@ static gboolean gst_multi_file_sink_event (GstBaseSink * sink,
 static GType
 gst_multi_file_sink_next_get_type (void)
 {
-  static GType multi_file_sink_next_type = 0;
+  static GType multi_file_sync_next_type = 0;
   static const GEnumValue next_types[] = {
     {GST_MULTI_FILE_SINK_NEXT_BUFFER, "New file for each buffer", "buffer"},
     {GST_MULTI_FILE_SINK_NEXT_DISCONT, "New file after each discontinuity",
@@ -143,19 +181,15 @@ gst_multi_file_sink_next_get_type (void)
     {GST_MULTI_FILE_SINK_NEXT_MAX_SIZE, "New file when the configured maximum "
           "file size would be exceeded with the next buffer or buffer list",
         "max-size"},
-    {GST_MULTI_FILE_SINK_NEXT_MAX_DURATION,
-          "New file when the configured maximum "
-          "file duration would be exceeded with the next buffer or buffer list",
-        "max-duration"},
     {0, NULL, NULL}
   };
 
-  if (!multi_file_sink_next_type) {
-    multi_file_sink_next_type =
+  if (!multi_file_sync_next_type) {
+    multi_file_sync_next_type =
         g_enum_register_static ("GstMultiFileSinkNext", next_types);
   }
 
-  return multi_file_sink_next_type;
+  return multi_file_sync_next_type;
 }
 
 #define gst_multi_file_sink_parent_class parent_class
@@ -183,31 +217,37 @@ gst_multi_file_sink_class_init (GstMultiFileSinkClass * klass)
           0, G_MAXINT, DEFAULT_INDEX,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
-   * GstMultiFileSink:post-messages:
+   * GstMultiFileSink:post-messages
    *
    * Post a message on the GstBus for each file.
+   *
+   * Since: 0.10.17
    */
   g_object_class_install_property (gobject_class, PROP_POST_MESSAGES,
       g_param_spec_boolean ("post-messages", "Post Messages",
           "Post a message for each file with information of the buffer",
           DEFAULT_POST_MESSAGES, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
-   * GstMultiFileSink:next-file:
+   * GstMultiFileSink:next-file
    *
    * When to start a new file.
+   *
+   * Since: 0.10.17
    */
   g_object_class_install_property (gobject_class, PROP_NEXT_FILE,
       g_param_spec_enum ("next-file", "Next File",
           "When to start a new file",
           GST_TYPE_MULTI_FILE_SINK_NEXT, DEFAULT_NEXT_FILE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_STATIC_STRINGS));
 
 
   /**
-   * GstMultiFileSink:max-files:
+   * GstMultiFileSink:max-files
    *
    * Maximum number of files to keep on disk. Once the maximum is reached, old
    * files start to be deleted to make room for new ones.
+   *
+   * Since: 0.10.31
    */
   g_object_class_install_property (gobject_class, PROP_MAX_FILES,
       g_param_spec_uint ("max-files", "Max files",
@@ -217,9 +257,11 @@ gst_multi_file_sink_class_init (GstMultiFileSinkClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstMultiFileSink:max-file-size:
+   * GstMultiFileSink:max-file-size
    *
    * Maximum file size before starting a new file in max-size mode.
+   *
+   * Since: 0.10.31
    */
   g_object_class_install_property (gobject_class, PROP_MAX_FILE_SIZE,
       g_param_spec_uint64 ("max-file-size", "Maximum File Size",
@@ -227,37 +269,9 @@ gst_multi_file_sink_class_init (GstMultiFileSinkClass * klass)
           0, G_MAXUINT64, DEFAULT_MAX_FILE_SIZE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  /**
-   * GstMultiFileSink:max-file-duration:
-   *
-   * Maximum file size before starting a new file in max-size mode.
-   */
-  g_object_class_install_property (gobject_class, PROP_MAX_FILE_DURATION,
-      g_param_spec_uint64 ("max-file-duration", "Maximum File Duration",
-          "Maximum file duration before starting a new file in max-duration mode "
-          "(in nanoseconds)", 0, G_MAXUINT64, DEFAULT_MAX_FILE_DURATION,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
-  /**
-   * GstMultiFileSink:aggregate-gops:
-   *
-   * Whether to aggregate complete GOPs before doing any processing. Set this
-   * to TRUE to make sure each new file starts with a keyframe. This requires
-   * the upstream element to flag buffers containing key units and delta
-   * units correctly. At least the MPEG-PS and MPEG-TS muxers should be doing
-   * this.
-   *
-   * Since: 1.6
-   */
-  g_object_class_install_property (gobject_class, PROP_AGGREGATE_GOPS,
-      g_param_spec_boolean ("aggregate-gops", "Aggregate GOPs",
-          "Whether to aggregate GOPs and process them as a whole without "
-          "splitting", DEFAULT_AGGREGATE_GOPS,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
   gobject_class->finalize = gst_multi_file_sink_finalize;
 
-  gstbasesink_class->start = GST_DEBUG_FUNCPTR (gst_multi_file_sink_start);
+  gstbasesink_class->get_times = NULL;
   gstbasesink_class->stop = GST_DEBUG_FUNCPTR (gst_multi_file_sink_stop);
   gstbasesink_class->render = GST_DEBUG_FUNCPTR (gst_multi_file_sink_render);
   gstbasesink_class->render_list =
@@ -269,13 +283,12 @@ gst_multi_file_sink_class_init (GstMultiFileSinkClass * klass)
   GST_DEBUG_CATEGORY_INIT (gst_multi_file_sink_debug, "multifilesink", 0,
       "multifilesink element");
 
-  gst_element_class_add_static_pad_template (gstelement_class, &sinktemplate);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sinktemplate));
   gst_element_class_set_static_metadata (gstelement_class, "Multi-File Sink",
       "Sink/File",
       "Write buffers to a sequentially named set of files",
       "David Schleef <ds@schleef.org>");
-
-  gst_type_mark_as_plugin_api (GST_TYPE_MULTI_FILE_SINK_NEXT, 0);
 }
 
 static void
@@ -286,10 +299,8 @@ gst_multi_file_sink_init (GstMultiFileSink * multifilesink)
   multifilesink->post_messages = DEFAULT_POST_MESSAGES;
   multifilesink->max_files = DEFAULT_MAX_FILES;
   multifilesink->max_file_size = DEFAULT_MAX_FILE_SIZE;
-  multifilesink->max_file_duration = DEFAULT_MAX_FILE_DURATION;
-
-  multifilesink->aggregate_gops = DEFAULT_AGGREGATE_GOPS;
-  multifilesink->gop_adapter = NULL;
+  multifilesink->files = NULL;
+  multifilesink->n_files = 0;
 
   gst_base_sink_set_sync (GST_BASE_SINK (multifilesink), FALSE);
 
@@ -303,6 +314,8 @@ gst_multi_file_sink_finalize (GObject * object)
   GstMultiFileSink *sink = GST_MULTI_FILE_SINK (object);
 
   g_free (sink->filename);
+  g_slist_foreach (sink->files, (GFunc) g_free, NULL);
+  g_slist_free (sink->files);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -343,12 +356,6 @@ gst_multi_file_sink_set_property (GObject * object, guint prop_id,
     case PROP_MAX_FILE_SIZE:
       sink->max_file_size = g_value_get_uint64 (value);
       break;
-    case PROP_MAX_FILE_DURATION:
-      sink->max_file_duration = g_value_get_uint64 (value);
-      break;
-    case PROP_AGGREGATE_GOPS:
-      sink->aggregate_gops = g_value_get_boolean (value);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -380,33 +387,12 @@ gst_multi_file_sink_get_property (GObject * object, guint prop_id,
     case PROP_MAX_FILE_SIZE:
       g_value_set_uint64 (value, sink->max_file_size);
       break;
-    case PROP_MAX_FILE_DURATION:
-      g_value_set_uint64 (value, sink->max_file_duration);
-      break;
-    case PROP_AGGREGATE_GOPS:
-      g_value_set_boolean (value, sink->aggregate_gops);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
-static gboolean
-gst_multi_file_sink_start (GstBaseSink * bsink)
-{
-  GstMultiFileSink *sink = GST_MULTI_FILE_SINK (bsink);
-
-  if (sink->aggregate_gops)
-    sink->gop_adapter = gst_adapter_new ();
-  sink->potential_next_gop = NULL;
-  sink->file_pts = GST_CLOCK_TIME_NONE;
-
-  g_queue_init (&sink->old_files);
-
-  return TRUE;
-}
-
 static gboolean
 gst_multi_file_sink_stop (GstBaseSink * sink)
 {
@@ -428,22 +414,8 @@ gst_multi_file_sink_stop (GstBaseSink * sink)
     multifilesink->streamheaders = NULL;
   }
 
-  if (multifilesink->gop_adapter != NULL) {
-    g_object_unref (multifilesink->gop_adapter);
-    multifilesink->gop_adapter = NULL;
-  }
-
-  if (multifilesink->potential_next_gop != NULL) {
-    g_list_free_full (multifilesink->potential_next_gop,
-        (GDestroyNotify) gst_buffer_unref);
-    multifilesink->potential_next_gop = NULL;
-  }
-
   multifilesink->force_key_unit_count = -1;
 
-  g_queue_foreach (&multifilesink->old_files, (GFunc) g_free, NULL);
-  g_queue_clear (&multifilesink->old_files);
-
   return TRUE;
 }
 
@@ -473,30 +445,6 @@ gst_multi_file_sink_post_message_full (GstMultiFileSink * multifilesink,
       gst_message_new_element (GST_OBJECT_CAST (multifilesink), s));
 }
 
-static void
-gst_multi_file_sink_post_message_from_time (GstMultiFileSink * multifilesink,
-    GstClockTime timestamp, GstClockTime duration, const char *filename)
-{
-  GstClockTime running_time, stream_time;
-  guint64 offset, offset_end;
-  GstSegment *segment;
-  GstFormat format;
-
-  if (!multifilesink->post_messages)
-    return;
-
-  segment = &GST_BASE_SINK (multifilesink)->segment;
-  format = segment->format;
-
-  offset = -1;
-  offset_end = -1;
-
-  running_time = gst_segment_to_running_time (segment, format, timestamp);
-  stream_time = gst_segment_to_stream_time (segment, format, timestamp);
-
-  gst_multi_file_sink_post_message_full (multifilesink, timestamp, duration,
-      offset, offset_end, running_time, stream_time, filename);
-}
 
 static void
 gst_multi_file_sink_post_message (GstMultiFileSink * multifilesink,
@@ -557,9 +505,9 @@ gst_multi_file_sink_write_stream_headers (GstMultiFileSink * sink)
 }
 
 static GstFlowReturn
-gst_multi_file_sink_write_buffer (GstMultiFileSink * multifilesink,
-    GstBuffer * buffer)
+gst_multi_file_sink_render (GstBaseSink * sink, GstBuffer * buffer)
 {
+  GstMultiFileSink *multifilesink;
   GstMapInfo map;
   gchar *filename;
   gboolean ret;
@@ -568,6 +516,8 @@ gst_multi_file_sink_write_buffer (GstMultiFileSink * multifilesink,
 
   gst_buffer_map (buffer, &map, GST_MAP_READ);
 
+  multifilesink = GST_MULTI_FILE_SINK (sink);
+
   switch (multifilesink->next_file) {
     case GST_MULTI_FILE_SINK_NEXT_BUFFER:
       gst_multi_file_sink_ensure_max_files (multifilesink);
@@ -578,10 +528,10 @@ gst_multi_file_sink_write_buffer (GstMultiFileSink * multifilesink,
       if (!ret)
         goto write_error;
 
+      multifilesink->files = g_slist_append (multifilesink->files, filename);
+      multifilesink->n_files += 1;
+
       gst_multi_file_sink_post_message (multifilesink, buffer, filename);
-
-      gst_multi_file_sink_add_old_file (multifilesink, filename);
-
       multifilesink->index++;
 
       break;
@@ -636,13 +586,11 @@ gst_multi_file_sink_write_buffer (GstMultiFileSink * multifilesink,
       if (multifilesink->file == NULL) {
         if (!gst_multi_file_sink_open_next_file (multifilesink))
           goto stdio_write_error;
-
-        /* we don't need to write stream headers here, they will be inserted in
-         * the stream by upstream elements if key unit events have
-         * all_headers=true set
-         */
       }
 
+      if (!gst_multi_file_sink_write_stream_headers (multifilesink))
+        goto stdio_write_error;
+
       ret = fwrite (map.data, map.size, 1, multifilesink->file);
 
       if (ret != 1)
@@ -662,7 +610,7 @@ gst_multi_file_sink_write_buffer (GstMultiFileSink * multifilesink,
 
         if (multifilesink->file != NULL) {
           first_file = FALSE;
-          gst_multi_file_sink_close_file (multifilesink, buffer);
+          gst_multi_file_sink_close_file (multifilesink, NULL);
         }
       }
 
@@ -682,46 +630,6 @@ gst_multi_file_sink_write_buffer (GstMultiFileSink * multifilesink,
       multifilesink->cur_file_size += map.size;
       break;
     }
-    case GST_MULTI_FILE_SINK_NEXT_MAX_DURATION:{
-      GstClockTime new_duration = 0;
-
-      if (GST_BUFFER_PTS_IS_VALID (buffer)
-          && GST_CLOCK_TIME_IS_VALID (multifilesink->file_pts)) {
-        /* The new duration will extend to this new buffer pts ... */
-        new_duration = GST_BUFFER_PTS (buffer) - multifilesink->file_pts;
-        /* ... and duration (if it has one) */
-        if (GST_BUFFER_DURATION_IS_VALID (buffer))
-          new_duration += GST_BUFFER_DURATION (buffer);
-      }
-
-      if (new_duration > multifilesink->max_file_duration) {
-
-        GST_INFO_OBJECT (multifilesink,
-            "new_duration: %" G_GUINT64_FORMAT ", max. duration %"
-            G_GUINT64_FORMAT, new_duration, multifilesink->max_file_duration);
-
-        if (multifilesink->file != NULL) {
-          first_file = FALSE;
-          gst_multi_file_sink_close_file (multifilesink, buffer);
-        }
-      }
-
-      if (multifilesink->file == NULL) {
-        if (!gst_multi_file_sink_open_next_file (multifilesink))
-          goto stdio_write_error;
-
-        multifilesink->file_pts = GST_BUFFER_PTS (buffer);
-        if (!first_file)
-          gst_multi_file_sink_write_stream_headers (multifilesink);
-      }
-
-      ret = fwrite (map.data, map.size, 1, multifilesink->file);
-
-      if (ret != 1)
-        goto stdio_write_error;
-
-      break;
-    }
     default:
       g_assert_not_reached ();
   }
@@ -764,74 +672,17 @@ stdio_write_error:
   return GST_FLOW_ERROR;
 }
 
-static GstFlowReturn
-gst_multi_file_sink_render (GstBaseSink * bsink, GstBuffer * buffer)
+static gboolean
+buffer_list_calc_size (GstBuffer ** buf, guint idx, gpointer data)
 {
-  GstMultiFileSink *sink = GST_MULTI_FILE_SINK (bsink);
-  GstFlowReturn flow = GST_FLOW_OK;
-  gboolean key_unit, header;
+  guint *p_size = data;
+  gsize buf_size;
 
-  header = GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_HEADER);
-  key_unit = !GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DELTA_UNIT);
+  buf_size = gst_buffer_get_size (*buf);
+  GST_TRACE ("buffer %u has size %" G_GSIZE_FORMAT, idx, buf_size);
+  *p_size += buf_size;
 
-  if (sink->aggregate_gops) {
-    GstBuffer *gop_buffer = NULL;
-    guint avail;
-
-    avail = gst_adapter_available (sink->gop_adapter);
-
-    GST_LOG_OBJECT (sink, "aggregate GOP: received %s%s unit buffer: "
-        "%" GST_PTR_FORMAT,
-        (key_unit) ? "key" : "delta", (header) ? " header" : "", buffer);
-
-    /* If it's a header buffer, it might potentially be for the next GOP */
-    if (header) {
-      GST_LOG_OBJECT (sink, "Accumulating buffer to potential next GOP");
-      sink->potential_next_gop =
-          g_list_append (sink->potential_next_gop, gst_buffer_ref (buffer));
-    } else {
-      if (key_unit && avail > 0) {
-        GstClockTime pts, dts;
-        GST_LOG_OBJECT (sink, "Grabbing pending completed GOP");
-        pts = gst_adapter_prev_pts_at_offset (sink->gop_adapter, 0, NULL);
-        dts = gst_adapter_prev_dts_at_offset (sink->gop_adapter, 0, NULL);
-        gop_buffer = gst_adapter_take_buffer (sink->gop_adapter, avail);
-        GST_BUFFER_PTS (gop_buffer) = pts;
-        GST_BUFFER_DTS (gop_buffer) = dts;
-      }
-
-      /* just accumulate the buffer */
-      if (sink->potential_next_gop) {
-        GList *tmp;
-        GST_LOG_OBJECT (sink,
-            "Carrying over pending next GOP data into adapter");
-        /* If we have pending data, put that first in the adapter */
-        for (tmp = sink->potential_next_gop; tmp; tmp = tmp->next) {
-          GstBuffer *tmpb = (GstBuffer *) tmp->data;
-          gst_adapter_push (sink->gop_adapter, tmpb);
-        }
-        g_list_free (sink->potential_next_gop);
-        sink->potential_next_gop = NULL;
-      }
-      GST_LOG_OBJECT (sink, "storing buffer in adapter");
-      gst_adapter_push (sink->gop_adapter, gst_buffer_ref (buffer));
-
-      if (gop_buffer != NULL) {
-        GST_DEBUG_OBJECT (sink, "writing out pending GOP, %u bytes", avail);
-        GST_DEBUG_OBJECT (sink,
-            "gop buffer pts:%" GST_TIME_FORMAT " dts:%" GST_TIME_FORMAT
-            " duration:%" GST_TIME_FORMAT,
-            GST_TIME_ARGS (GST_BUFFER_PTS (gop_buffer)),
-            GST_TIME_ARGS (GST_BUFFER_DTS (gop_buffer)),
-            GST_TIME_ARGS (GST_BUFFER_DURATION (gop_buffer)));
-        flow = gst_multi_file_sink_write_buffer (sink, gop_buffer);
-        gst_buffer_unref (gop_buffer);
-      }
-    }
-  } else {
-    flow = gst_multi_file_sink_write_buffer (sink, buffer);
-  }
-  return flow;
+  return TRUE;
 }
 
 static gboolean
@@ -861,9 +712,9 @@ static GstFlowReturn
 gst_multi_file_sink_render_list (GstBaseSink * sink, GstBufferList * list)
 {
   GstBuffer *buf;
-  guint size;
+  guint size = 0;
 
-  size = gst_buffer_list_calculate_size (list);
+  gst_buffer_list_foreach (list, buffer_list_calc_size, &size);
   GST_LOG_OBJECT (sink, "total size of buffer list %p: %u", list, size);
 
   /* copy all buffers in the list into one single buffer, so we can use
@@ -917,33 +768,19 @@ gst_multi_file_sink_set_caps (GstBaseSink * sink, GstCaps * caps)
   return TRUE;
 }
 
-/* Takes ownership of the filename string */
-static void
-gst_multi_file_sink_add_old_file (GstMultiFileSink * multifilesink, gchar * fn)
-{
-  /* Only add file to the list if a max_files limit is set, otherwise we never
-   * prune the list and memory just builds up until the pipeline is stopped. */
-  if (multifilesink->max_files > 0) {
-    g_queue_push_tail (&multifilesink->old_files, fn);
-  } else {
-    g_free (fn);
-  }
-}
-
 static void
 gst_multi_file_sink_ensure_max_files (GstMultiFileSink * multifilesink)
 {
-  guint max_files = multifilesink->max_files;
+  char *filename;
 
-  if (max_files == 0)
-    return;
-
-  while (g_queue_get_length (&multifilesink->old_files) >= max_files) {
-    gchar *filename;
-
-    filename = g_queue_pop_head (&multifilesink->old_files);
+  while (multifilesink->max_files &&
+      multifilesink->n_files >= multifilesink->max_files) {
+    filename = multifilesink->files->data;
     g_remove (filename);
     g_free (filename);
+    multifilesink->files = g_slist_delete_link (multifilesink->files,
+        multifilesink->files);
+    multifilesink->n_files -= 1;
   }
 }
 
@@ -982,12 +819,13 @@ gst_multi_file_sink_event (GstBaseSink * sink, GstEvent * event)
         offset = offset_end = -1;
         filename = g_strdup_printf (multifilesink->filename,
             multifilesink->index);
+        gst_multi_file_sink_post_message_full (multifilesink, timestamp,
+            duration, offset, offset_end, running_time, stream_time, filename);
+
+        g_free (filename);
 
         gst_multi_file_sink_close_file (multifilesink, NULL);
 
-        gst_multi_file_sink_post_message_full (multifilesink, timestamp,
-            duration, offset, offset_end, running_time, stream_time, filename);
-        g_free (filename);
       }
 
       if (multifilesink->file == NULL) {
@@ -997,29 +835,6 @@ gst_multi_file_sink_event (GstBaseSink * sink, GstEvent * event)
 
       break;
     }
-    case GST_EVENT_EOS:
-      if (multifilesink->aggregate_gops) {
-        GstBuffer *buf = gst_buffer_new ();
-
-        /* push key unit buffer to force writing out the pending GOP data */
-        GST_INFO_OBJECT (sink, "EOS, write pending GOP data");
-        GST_BUFFER_FLAG_UNSET (buf, GST_BUFFER_FLAG_DELTA_UNIT);
-        gst_multi_file_sink_render (sink, buf);
-        gst_buffer_unref (buf);
-      }
-      if (multifilesink->file) {
-        gchar *filename;
-
-        filename = g_strdup_printf (multifilesink->filename,
-            multifilesink->index);
-
-        gst_multi_file_sink_close_file (multifilesink, NULL);
-
-        gst_multi_file_sink_post_message_from_time (multifilesink,
-            GST_BASE_SINK (multifilesink)->segment.position, -1, filename);
-        g_free (filename);
-      }
-      break;
     default:
       break;
   }
@@ -1045,7 +860,6 @@ gst_multi_file_sink_open_next_file (GstMultiFileSink * multifilesink)
   g_return_val_if_fail (multifilesink->file == NULL, FALSE);
 
   gst_multi_file_sink_ensure_max_files (multifilesink);
-
   filename = g_strdup_printf (multifilesink->filename, multifilesink->index);
   multifilesink->file = g_fopen (filename, "wb");
   if (multifilesink->file == NULL) {
@@ -1054,8 +868,8 @@ gst_multi_file_sink_open_next_file (GstMultiFileSink * multifilesink)
   }
 
   GST_INFO_OBJECT (multifilesink, "opening file %s", filename);
-
-  gst_multi_file_sink_add_old_file (multifilesink, filename);
+  multifilesink->files = g_slist_append (multifilesink->files, filename);
+  multifilesink->n_files += 1;
 
   multifilesink->cur_file_size = 0;
   return TRUE;
