@@ -22,11 +22,10 @@
 
 /**
  * SECTION:element-rganalysis
- * @title: rganalysis
  * @see_also: #GstRgVolume
  *
  * This element analyzes raw audio sample data in accordance with the proposed
- * [ReplayGain standard](https://wiki.hydrogenaud.io/index.php?title=ReplayGain) for
+ * <ulink url="http://replaygain.org">ReplayGain standard</ulink> for
  * calculating the ideal replay gain for music tracks and albums.  The element
  * is designed as a pass-through filter that never modifies any data.  As it
  * receives an EOS event, it finalizes the ongoing analysis and generates a tag
@@ -34,17 +33,18 @@
  * posted on the message bus with a tag message.  The EOS event is forwarded as
  * normal afterwards.  Result tag lists at least contain the tags
  * #GST_TAG_TRACK_GAIN, #GST_TAG_TRACK_PEAK and #GST_TAG_REFERENCE_LEVEL.
- *
+ * 
  * Because the generated metadata tags become available at the end of streams,
  * downstream muxer and encoder elements are normally unable to save them in
  * their output since they generally save metadata in the file header.
  * Therefore, it is often necessary that applications read the results in a bus
  * event handler for the tag message.  Obtaining the values this way is always
- * needed for album processing (see #GstRgAnalysis:num-tracks property) since
- * the album gain and peak values need to be associated with all tracks of an
- * album, not just the last one.
- *
- * ## Example launch lines
+ * needed for <link linkend="GstRgAnalysis--num-tracks">album processing</link>
+ * since the album gain and peak values need to be associated with all tracks of
+ * an album, not just the last one.
+ * 
+ * <refsect2>
+ * <title>Example launch lines</title>
  * |[
  * gst-launch-1.0 -t audiotestsrc wave=sine num-buffers=512 ! rganalysis ! fakesink
  * ]| Analyze a simple test waveform
@@ -56,17 +56,21 @@
  * gst-launch-1.0 -t gnomevfssrc location=http://replaygain.hydrogenaudio.org/ref_pink.wav \
  *     ! wavparse ! rganalysis ! fakesink
  * ]| Analyze the pink noise reference file
- *
+ * <para>
  * The above launch line yields a result gain of +6 dB (instead of the expected
  * +0 dB).  This is not in error, refer to the #GstRgAnalysis:reference-level
  * property documentation for more information.
- *
- * ## Acknowledgements
- *
- * This element is based on code used in the [vorbisgain](https://sjeng.org/vorbisgain.html)
- * program and many others.  The relevant parts are copyrighted by David Robinson, Glen Sawyer
+ * </para>
+ * </refsect2>
+ * <refsect2>
+ * <title>Acknowledgements</title>
+ * <para>
+ * This element is based on code used in the <ulink
+ * url="http://sjeng.org/vorbisgain.html">vorbisgain</ulink> program and many
+ * others.  The relevant parts are copyrighted by David Robinson, Glen Sawyer
  * and Frank Klemm.
- *
+ * </para>
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -125,8 +129,6 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
 
 #define gst_rg_analysis_parent_class parent_class
 G_DEFINE_TYPE (GstRgAnalysis, gst_rg_analysis, GST_TYPE_BASE_TRANSFORM);
-GST_ELEMENT_REGISTER_DEFINE (rganalysis, "rganalysis", GST_RANK_NONE,
-    GST_TYPE_RG_ANALYSIS);
 
 static void gst_rg_analysis_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -167,7 +169,7 @@ gst_rg_analysis_class_init (GstRgAnalysisClass * klass)
    * GstRgAnalysis:num-tracks:
    *
    * Number of remaining album tracks.
-   *
+   * 
    * Analyzing several streams sequentially and assigning them a common result
    * gain is known as "album processing".  If this gain is used during playback
    * (by switching to "album mode"), all tracks of an album receive the same
@@ -207,7 +209,7 @@ gst_rg_analysis_class_init (GstRgAnalysisClass * klass)
    * skip the processing of streams that already contain the necessary tags.
    * Data will flow as usual but the element will not consume CPU time and will
    * not generate result tags.  To enable possible skipping, set this property
-   * to %FALSE.
+   * to #FALSE.
    *
    * If used in conjunction with <link linkend="GstRgAnalysis--num-tracks">album
    * processing</link>, the element will skip the number of remaining album
@@ -231,8 +233,10 @@ gst_rg_analysis_class_init (GstRgAnalysisClass * klass)
    * ReplayGain, set this property to 83.
    *
    * Almost all software uses 89 dB as a reference however, and this value has
-   * become the new official value, and that change has been acclaimed by the
-   * original author of the ReplayGain proposal.
+   * become the new official value.  That is to say, while the change has been
+   * acclaimed by the author of the ReplayGain proposal, the <ulink
+   * url="http://replaygain.org">webpage</ulink> is still outdated at the time
+   * of this writing.
    *
    * The value was changed because the original proposal recommends a default
    * pre-amp value of +6 dB for playback.  This seemed a bit odd, as it means
@@ -263,8 +267,10 @@ gst_rg_analysis_class_init (GstRgAnalysisClass * klass)
   trans_class->stop = GST_DEBUG_FUNCPTR (gst_rg_analysis_stop);
   trans_class->passthrough_on_same_caps = TRUE;
 
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
-  gst_element_class_add_static_pad_template (element_class, &sink_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_factory));
   gst_element_class_set_static_metadata (element_class, "ReplayGain analysis",
       "Filter/Analyzer/Audio",
       "Perform the ReplayGain analysis",
@@ -526,7 +532,6 @@ gst_rg_analysis_stop (GstBaseTransform * base)
   return TRUE;
 }
 
-/* FIXME: handle global vs. stream-tags? */
 static void
 gst_rg_analysis_handle_tags (GstRgAnalysis * filter,
     const GstTagList * tag_list)
@@ -627,10 +632,9 @@ gst_rg_analysis_handle_eos (GstRgAnalysis * filter)
       GST_LOG_OBJECT (filter, "posting tag list with results");
       gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
           GST_TAG_REFERENCE_LEVEL, filter->reference_level, NULL);
-      /* This takes ownership of our reference to the list */
-      gst_pad_push_event (GST_BASE_TRANSFORM_SRC_PAD (filter),
-          gst_event_new_tag (tag_list));
-      tag_list = NULL;
+      /* This steals our reference to the list: */
+      gst_pad_push_event (GST_BASE_TRANSFORM_SRC_PAD (GST_BASE_TRANSFORM
+              (filter)), gst_event_new_tag (gst_tag_list_ref (tag_list)));
     }
   }
 
@@ -649,7 +653,6 @@ gst_rg_analysis_handle_eos (GstRgAnalysis * filter)
     g_object_notify (G_OBJECT (filter), "num-tracks");
 }
 
-/* FIXME: return tag list (lists?) based on input tags.. */
 static gboolean
 gst_rg_analysis_track_result (GstRgAnalysis * filter, GstTagList ** tag_list)
 {
