@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,9 +26,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <gst/rtp/gstrtpbuffer.h>
-#include <gst/audio/audio.h>
 #include "gstrtpsirendepay.h"
-#include "gstrtputils.h"
 
 static GstStaticPadTemplate gst_rtp_siren_depay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
@@ -36,27 +34,28 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"audio\", "
-        "clock-rate = (int) 16000, " "encoding-name = (string) \"SIREN\"")
-    /* This is the default, so the peer doesn't have to specify it */
-    /*  " "dct-length = (int) 320") */
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
+        "clock-rate = (int) 16000, "
+        "encoding-name = (string) \"SIREN\", " "dct-length = (int) 320")
     );
 
-     static GstStaticPadTemplate gst_rtp_siren_depay_src_template =
-         GST_STATIC_PAD_TEMPLATE ("src",
+static GstStaticPadTemplate gst_rtp_siren_depay_src_template =
+GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-siren, " "dct-length = (int) 320")
     );
 
-     static GstBuffer *gst_rtp_siren_depay_process (GstRTPBaseDepayload *
-    depayload, GstRTPBuffer * rtp);
-     static gboolean gst_rtp_siren_depay_setcaps (GstRTPBaseDepayload *
-    depayload, GstCaps * caps);
+static GstBuffer *gst_rtp_siren_depay_process (GstRTPBaseDepayload * depayload,
+    GstBuffer * buf);
+static gboolean gst_rtp_siren_depay_setcaps (GstRTPBaseDepayload * depayload,
+    GstCaps * caps);
 
 G_DEFINE_TYPE (GstRTPSirenDepay, gst_rtp_siren_depay,
     GST_TYPE_RTP_BASE_DEPAYLOAD);
 
-     static void gst_rtp_siren_depay_class_init (GstRTPSirenDepayClass * klass)
+static void
+gst_rtp_siren_depay_class_init (GstRTPSirenDepayClass * klass)
 {
   GstElementClass *gstelement_class;
   GstRTPBaseDepayloadClass *gstrtpbasedepayload_class;
@@ -64,13 +63,13 @@ G_DEFINE_TYPE (GstRTPSirenDepay, gst_rtp_siren_depay,
   gstelement_class = (GstElementClass *) klass;
   gstrtpbasedepayload_class = (GstRTPBaseDepayloadClass *) klass;
 
-  gstrtpbasedepayload_class->process_rtp_packet = gst_rtp_siren_depay_process;
+  gstrtpbasedepayload_class->process = gst_rtp_siren_depay_process;
   gstrtpbasedepayload_class->set_caps = gst_rtp_siren_depay_setcaps;
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_rtp_siren_depay_src_template);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_rtp_siren_depay_sink_template);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_rtp_siren_depay_src_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_rtp_siren_depay_sink_template));
   gst_element_class_set_static_metadata (gstelement_class,
       "RTP Siren packet depayloader", "Codec/Depayloader/Network/RTP",
       "Extracts Siren audio from RTP packets",
@@ -103,16 +102,14 @@ gst_rtp_siren_depay_setcaps (GstRTPBaseDepayload * depayload, GstCaps * caps)
 }
 
 static GstBuffer *
-gst_rtp_siren_depay_process (GstRTPBaseDepayload * depayload,
-    GstRTPBuffer * rtp)
+gst_rtp_siren_depay_process (GstRTPBaseDepayload * depayload, GstBuffer * buf)
 {
   GstBuffer *outbuf;
+  GstRTPBuffer rtp = { NULL };
 
-  outbuf = gst_rtp_buffer_get_payload_buffer (rtp);
-
-  if (outbuf) {
-    gst_rtp_drop_non_audio_meta (depayload, outbuf);
-  }
+  gst_rtp_buffer_map (buf, GST_MAP_READ, &rtp);
+  outbuf = gst_rtp_buffer_get_payload_buffer (&rtp);
+  gst_rtp_buffer_unmap (&rtp);
 
   return outbuf;
 }
