@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -22,13 +22,10 @@
 #endif
 
 #include <gst/rtp/gstrtpbuffer.h>
-#include <gst/video/video.h>
 
 #include <string.h>
 #include <stdlib.h>
-#include "gstrtpelements.h"
 #include "gstrtpvrawdepay.h"
-#include "gstrtputils.h"
 
 GST_DEBUG_CATEGORY_STATIC (rtpvrawdepay_debug);
 #define GST_CAT_DEFAULT (rtpvrawdepay_debug)
@@ -41,42 +38,23 @@ GST_STATIC_PAD_TEMPLATE ("src",
     );
 
 static GstStaticPadTemplate gst_rtp_vraw_depay_sink_template =
-    GST_STATIC_PAD_TEMPLATE ("sink",
+GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"video\", "
-        "clock-rate = (int) 90000, "
-        "encoding-name = (string) \"RAW\", "
-        "sampling = (string) { \"RGB\", \"RGBA\", \"BGR\", \"BGRA\", "
-        "\"YCbCr-4:4:4\", \"YCbCr-4:2:2\", \"YCbCr-4:2:0\", "
-        "\"YCbCr-4:1:1\" },"
-        /* we cannot express these as strings 
-         * "width = (string) [1 32767],"
-         * "height = (string) [1 32767],"
-         */
-        "depth = (string) 8; "
-        "application/x-rtp, "
-        "media = (string) video, "
-        "clock-rate = (int) 90000, "
-        "encoding-name = (string) RAW, sampling = (string) YCbCr-4:2:2,"
-        /* we cannot express these as strings
-         * "width = (string) [1 32767],"
-         * "height = (string) [1 32767],"
-         */
-        "depth = (string) 10")
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
+        "clock-rate = (int) 90000, " "encoding-name = (string) \"RAW\"")
     );
 
 #define gst_rtp_vraw_depay_parent_class parent_class
 G_DEFINE_TYPE (GstRtpVRawDepay, gst_rtp_vraw_depay,
     GST_TYPE_RTP_BASE_DEPAYLOAD);
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtpvrawdepay, "rtpvrawdepay",
-    GST_RANK_SECONDARY, GST_TYPE_RTP_VRAW_DEPAY, rtp_element_init (plugin));
 
 static gboolean gst_rtp_vraw_depay_setcaps (GstRTPBaseDepayload * depayload,
     GstCaps * caps);
-static GstBuffer *gst_rtp_vraw_depay_process_packet (GstRTPBaseDepayload *
-    depay, GstRTPBuffer * rtp);
+static GstBuffer *gst_rtp_vraw_depay_process (GstRTPBaseDepayload * depayload,
+    GstBuffer * buf);
 
 static GstStateChangeReturn gst_rtp_vraw_depay_change_state (GstElement *
     element, GstStateChange transition);
@@ -96,14 +74,13 @@ gst_rtp_vraw_depay_class_init (GstRtpVRawDepayClass * klass)
   gstelement_class->change_state = gst_rtp_vraw_depay_change_state;
 
   gstrtpbasedepayload_class->set_caps = gst_rtp_vraw_depay_setcaps;
-  gstrtpbasedepayload_class->process_rtp_packet =
-      gst_rtp_vraw_depay_process_packet;
+  gstrtpbasedepayload_class->process = gst_rtp_vraw_depay_process;
   gstrtpbasedepayload_class->handle_event = gst_rtp_vraw_depay_handle_event;
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_rtp_vraw_depay_src_template);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_rtp_vraw_depay_sink_template);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_rtp_vraw_depay_src_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_rtp_vraw_depay_sink_template));
 
   gst_element_class_set_static_metadata (gstelement_class,
       "RTP Raw Video depayloader", "Codec/Depayloader/Network/RTP",
@@ -120,16 +97,14 @@ gst_rtp_vraw_depay_init (GstRtpVRawDepay * rtpvrawdepay)
 }
 
 static void
-gst_rtp_vraw_depay_reset (GstRtpVRawDepay * rtpvrawdepay, gboolean full)
+gst_rtp_vraw_depay_reset (GstRtpVRawDepay * rtpvrawdepay)
 {
   if (rtpvrawdepay->outbuf) {
-    gst_video_frame_unmap (&rtpvrawdepay->frame);
     gst_buffer_unref (rtpvrawdepay->outbuf);
     rtpvrawdepay->outbuf = NULL;
   }
   rtpvrawdepay->timestamp = -1;
-
-  if (full && rtpvrawdepay->pool) {
+  if (rtpvrawdepay->pool) {
     gst_buffer_pool_set_active (rtpvrawdepay->pool, FALSE);
     gst_object_unref (rtpvrawdepay->pool);
     rtpvrawdepay->pool = NULL;
@@ -165,11 +140,6 @@ gst_rtp_vraw_depay_negotiate_pool (GstRtpVRawDepay * depay, GstCaps * caps,
   if (pool == NULL) {
     /* we did not get a pool, make one ourselves then */
     pool = gst_video_buffer_pool_new ();
-    {
-      gchar *name = g_strdup_printf ("%s-pool", GST_OBJECT_NAME (depay));
-      g_object_set (pool, "name", name, NULL);
-      g_free (name);
-    }
   }
 
   if (depay->pool)
@@ -202,7 +172,7 @@ gst_rtp_vraw_depay_setcaps (GstRTPBaseDepayload * depayload, GstCaps * caps)
   GstRtpVRawDepay *rtpvrawdepay;
   gint clock_rate;
   const gchar *str;
-  gint format, width, height, depth, pgroup, xinc, yinc;
+  gint format, width, height, pgroup, xinc, yinc;
   GstCaps *srccaps;
   gboolean res;
   GstFlowReturn ret;
@@ -225,10 +195,6 @@ gst_rtp_vraw_depay_setcaps (GstRTPBaseDepayload * depayload, GstCaps * caps)
     goto no_height;
   height = atoi (str);
 
-  if (!(str = gst_structure_get_string (structure, "depth")))
-    goto no_depth;
-  depth = atoi (str);
-
   /* optional interlace value but we don't handle interlaced
    * formats yet */
   if (gst_structure_get_string (structure, "interlace"))
@@ -253,14 +219,8 @@ gst_rtp_vraw_depay_setcaps (GstRTPBaseDepayload * depayload, GstCaps * caps)
     format = GST_VIDEO_FORMAT_AYUV;
     pgroup = 3;
   } else if (!strcmp (str, "YCbCr-4:2:2")) {
-    if (depth == 8) {
-      format = GST_VIDEO_FORMAT_UYVY;
-      pgroup = 4;
-    } else if (depth == 10) {
-      format = GST_VIDEO_FORMAT_UYVP;
-      pgroup = 5;
-    } else
-      goto unknown_format;
+    format = GST_VIDEO_FORMAT_UYVY;
+    pgroup = 4;
     xinc = 2;
   } else if (!strcmp (str, "YCbCr-4:2:0")) {
     format = GST_VIDEO_FORMAT_I420;
@@ -270,9 +230,8 @@ gst_rtp_vraw_depay_setcaps (GstRTPBaseDepayload * depayload, GstCaps * caps)
     format = GST_VIDEO_FORMAT_Y41B;
     pgroup = 6;
     xinc = 4;
-  } else {
+  } else
     goto unknown_format;
-  }
 
   gst_video_info_init (&rtpvrawdepay->vinfo);
   gst_video_info_set_format (&rtpvrawdepay->vinfo, format, width, height);
@@ -310,11 +269,6 @@ no_height:
     GST_ERROR_OBJECT (depayload, "no height specified");
     return FALSE;
   }
-no_depth:
-  {
-    GST_ERROR_OBJECT (depayload, "no depth specified");
-    return FALSE;
-  }
 interlaced:
   {
     GST_ERROR_OBJECT (depayload, "interlaced formats not supported yet");
@@ -338,30 +292,29 @@ no_bufferpool:
 }
 
 static GstBuffer *
-gst_rtp_vraw_depay_process_packet (GstRTPBaseDepayload * depayload,
-    GstRTPBuffer * rtp)
+gst_rtp_vraw_depay_process (GstRTPBaseDepayload * depayload, GstBuffer * buf)
 {
   GstRtpVRawDepay *rtpvrawdepay;
-  guint8 *payload, *p0, *yp, *up, *vp, *headers;
+  guint8 *payload, *yp, *up, *vp, *headers;
   guint32 timestamp;
   guint cont, ystride, uvstride, pgroup, payload_len;
   gint width, height, xinc, yinc;
-  GstVideoFrame *frame;
-  gboolean marker;
-  GstBuffer *outbuf = NULL;
+  GstRTPBuffer rtp = { NULL };
+  GstVideoFrame frame;
 
   rtpvrawdepay = GST_RTP_VRAW_DEPAY (depayload);
 
-  timestamp = gst_rtp_buffer_get_timestamp (rtp);
+  gst_rtp_buffer_map (buf, GST_MAP_READ, &rtp);
+
+  timestamp = gst_rtp_buffer_get_timestamp (&rtp);
 
   if (timestamp != rtpvrawdepay->timestamp || rtpvrawdepay->outbuf == NULL) {
-    GstBuffer *new_buffer;
+    GstBuffer *outbuf;
     GstFlowReturn ret;
 
     GST_LOG_OBJECT (depayload, "new frame with timestamp %u", timestamp);
     /* new timestamp, flush old buffer and create new output buffer */
     if (rtpvrawdepay->outbuf) {
-      gst_video_frame_unmap (&rtpvrawdepay->frame);
       gst_rtp_base_depayload_push (depayload, rtpvrawdepay->outbuf);
       rtpvrawdepay->outbuf = NULL;
     }
@@ -376,37 +329,28 @@ gst_rtp_vraw_depay_process_packet (GstRTPBaseDepayload * depayload,
       gst_caps_unref (caps);
     }
 
-    ret =
-        gst_buffer_pool_acquire_buffer (rtpvrawdepay->pool, &new_buffer, NULL);
-
+    ret = gst_buffer_pool_acquire_buffer (rtpvrawdepay->pool, &outbuf, NULL);
     if (G_UNLIKELY (ret != GST_FLOW_OK))
       goto alloc_failed;
 
     /* clear timestamp from alloc... */
-    GST_BUFFER_PTS (new_buffer) = -1;
+    GST_BUFFER_TIMESTAMP (outbuf) = -1;
 
-    if (!gst_video_frame_map (&rtpvrawdepay->frame, &rtpvrawdepay->vinfo,
-            new_buffer, GST_MAP_WRITE | GST_VIDEO_FRAME_MAP_FLAG_NO_REF)) {
-      gst_buffer_unref (new_buffer);
-      goto invalid_frame;
-    }
-
-    rtpvrawdepay->outbuf = new_buffer;
+    rtpvrawdepay->outbuf = outbuf;
     rtpvrawdepay->timestamp = timestamp;
   }
 
-  frame = &rtpvrawdepay->frame;
-
-  g_assert (frame->buffer != NULL);
+  if (!gst_video_frame_map (&frame, &rtpvrawdepay->vinfo, rtpvrawdepay->outbuf,
+          GST_MAP_WRITE))
+    goto invalid_frame;
 
   /* get pointer and strides of the planes */
-  p0 = GST_VIDEO_FRAME_PLANE_DATA (frame, 0);
-  yp = GST_VIDEO_FRAME_COMP_DATA (frame, 0);
-  up = GST_VIDEO_FRAME_COMP_DATA (frame, 1);
-  vp = GST_VIDEO_FRAME_COMP_DATA (frame, 2);
+  yp = GST_VIDEO_FRAME_COMP_DATA (&frame, 0);
+  up = GST_VIDEO_FRAME_COMP_DATA (&frame, 1);
+  vp = GST_VIDEO_FRAME_COMP_DATA (&frame, 2);
 
-  ystride = GST_VIDEO_FRAME_COMP_STRIDE (frame, 0);
-  uvstride = GST_VIDEO_FRAME_COMP_STRIDE (frame, 1);
+  ystride = GST_VIDEO_FRAME_COMP_STRIDE (&frame, 0);
+  uvstride = GST_VIDEO_FRAME_COMP_STRIDE (&frame, 1);
 
   pgroup = rtpvrawdepay->pgroup;
   width = GST_VIDEO_INFO_WIDTH (&rtpvrawdepay->vinfo);
@@ -414,8 +358,8 @@ gst_rtp_vraw_depay_process_packet (GstRTPBaseDepayload * depayload,
   xinc = rtpvrawdepay->xinc;
   yinc = rtpvrawdepay->yinc;
 
-  payload = gst_rtp_buffer_get_payload (rtp);
-  payload_len = gst_rtp_buffer_get_payload_len (rtp);
+  payload = gst_rtp_buffer_get_payload (&rtp);
+  payload_len = gst_rtp_buffer_get_payload_len (&rtp);
 
   if (payload_len < 3)
     goto short_packet;
@@ -427,8 +371,6 @@ gst_rtp_vraw_depay_process_packet (GstRTPBaseDepayload * depayload,
   /* remember header position */
   headers = payload;
 
-  gst_rtp_copy_video_meta (rtpvrawdepay, frame->buffer, rtp->buffer);
-
   /* find data start */
   do {
     if (payload_len < 6)
@@ -473,7 +415,7 @@ gst_rtp_vraw_depay_process_packet (GstRTPBaseDepayload * depayload,
       goto next;
     }
 
-    /* calculate the maximum amount of bytes we can use per line */
+    /* calculate the maximim amount of bytes we can use per line */
     if (offs + ((length / pgroup) * xinc) > width) {
       plen = ((width - offs) * pgroup) / xinc;
       GST_WARNING_OBJECT (depayload, "clipping length %d, offset %d, plen %d",
@@ -491,10 +433,9 @@ gst_rtp_vraw_depay_process_packet (GstRTPBaseDepayload * depayload,
       case GST_VIDEO_FORMAT_BGR:
       case GST_VIDEO_FORMAT_BGRA:
       case GST_VIDEO_FORMAT_UYVY:
-      case GST_VIDEO_FORMAT_UYVP:
         /* samples are packed just like gstreamer packs them */
         offs /= xinc;
-        datap = p0 + (line * ystride) + (offs * pgroup);
+        datap = yp + (line * ystride) + (offs * pgroup);
 
         memcpy (datap, payload, plen);
         break;
@@ -503,7 +444,7 @@ gst_rtp_vraw_depay_process_packet (GstRTPBaseDepayload * depayload,
         gint i;
         guint8 *p;
 
-        datap = p0 + (line * ystride) + (offs * 4);
+        datap = yp + (line * ystride) + (offs * 4);
         p = payload;
 
         /* samples are packed in order Cb-Y-Cr for both interlaced and
@@ -581,42 +522,52 @@ gst_rtp_vraw_depay_process_packet (GstRTPBaseDepayload * depayload,
     payload_len -= length;
   }
 
-  marker = gst_rtp_buffer_get_marker (rtp);
+  gst_video_frame_unmap (&frame);
+  gst_rtp_buffer_unmap (&rtp);
 
-  if (marker) {
+  if (gst_rtp_buffer_get_marker (&rtp)) {
     GST_LOG_OBJECT (depayload, "marker, flushing frame");
-    gst_video_frame_unmap (&rtpvrawdepay->frame);
-    outbuf = rtpvrawdepay->outbuf;
-    rtpvrawdepay->outbuf = NULL;
+    if (rtpvrawdepay->outbuf) {
+      gst_rtp_base_depayload_push (depayload, rtpvrawdepay->outbuf);
+      rtpvrawdepay->outbuf = NULL;
+    }
     rtpvrawdepay->timestamp = -1;
   }
-  return outbuf;
+  return NULL;
 
   /* ERRORS */
 unknown_sampling:
   {
     GST_ELEMENT_ERROR (depayload, STREAM, FORMAT,
         (NULL), ("unimplemented sampling"));
+    gst_video_frame_unmap (&frame);
+    gst_rtp_buffer_unmap (&rtp);
     return NULL;
   }
 alloc_failed:
   {
     GST_WARNING_OBJECT (depayload, "failed to alloc output buffer");
+    gst_rtp_buffer_unmap (&rtp);
     return NULL;
   }
 invalid_frame:
   {
     GST_ERROR_OBJECT (depayload, "could not map video frame");
+    gst_rtp_buffer_unmap (&rtp);
     return NULL;
   }
 wrong_length:
   {
     GST_WARNING_OBJECT (depayload, "length not multiple of pgroup");
+    gst_video_frame_unmap (&frame);
+    gst_rtp_buffer_unmap (&rtp);
     return NULL;
   }
 short_packet:
   {
     GST_WARNING_OBJECT (depayload, "short packet");
+    gst_video_frame_unmap (&frame);
+    gst_rtp_buffer_unmap (&rtp);
     return NULL;
   }
 }
@@ -631,7 +582,7 @@ gst_rtp_vraw_depay_handle_event (GstRTPBaseDepayload * filter, GstEvent * event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_FLUSH_STOP:
-      gst_rtp_vraw_depay_reset (rtpvrawdepay, FALSE);
+      gst_rtp_vraw_depay_reset (rtpvrawdepay);
       break;
     default:
       break;
@@ -656,7 +607,7 @@ gst_rtp_vraw_depay_change_state (GstElement * element,
     case GST_STATE_CHANGE_NULL_TO_READY:
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      gst_rtp_vraw_depay_reset (rtpvrawdepay, TRUE);
+      gst_rtp_vraw_depay_reset (rtpvrawdepay);
       break;
     default:
       break;
@@ -666,7 +617,7 @@ gst_rtp_vraw_depay_change_state (GstElement * element,
 
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-      gst_rtp_vraw_depay_reset (rtpvrawdepay, TRUE);
+      gst_rtp_vraw_depay_reset (rtpvrawdepay);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
@@ -675,3 +626,10 @@ gst_rtp_vraw_depay_change_state (GstElement * element,
   }
   return ret;
 }
+
+gboolean
+gst_rtp_vraw_depay_plugin_init (GstPlugin * plugin)
+{
+  return gst_element_register (plugin, "rtpvrawdepay",
+      GST_RANK_SECONDARY, GST_TYPE_RTP_VRAW_DEPAY);
+}
