@@ -18,8 +18,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,7 +27,7 @@
 #endif
 
 #include "blend.h"
-#include "videomixerorc.h"
+#include "blendorc.h"
 
 #include <string.h>
 
@@ -253,7 +253,7 @@ blend_##format_name (GstVideoFrame * srcframe, gint xpos, gint ypos, \
   \
   info = srcframe->info.finfo; \
   dest_width = GST_VIDEO_FRAME_WIDTH (destframe); \
-  dest_height = GST_VIDEO_FRAME_HEIGHT (destframe); \
+  dest_height = GST_VIDEO_FRAME_WIDTH (destframe); \
   \
   xpos = x_round (xpos); \
   ypos = y_round (ypos); \
@@ -273,7 +273,7 @@ blend_##format_name (GstVideoFrame * srcframe, gint xpos, gint ypos, \
     ypos = 0; \
   } \
   /* If x or y offset are larger then the source it's outside of the picture */ \
-  if (xoffset > src_width || yoffset > src_height) { \
+  if (xoffset > src_width || yoffset > src_width) { \
     return; \
   } \
   \
@@ -498,7 +498,7 @@ blend_##format_name (GstVideoFrame * srcframe, gint xpos, gint ypos, \
   \
   info = srcframe->info.finfo; \
   dest_width = GST_VIDEO_FRAME_WIDTH (destframe); \
-  dest_height = GST_VIDEO_FRAME_HEIGHT (destframe); \
+  dest_height = GST_VIDEO_FRAME_WIDTH (destframe); \
   \
   xpos = GST_ROUND_UP_2 (xpos); \
   ypos = GST_ROUND_UP_2 (ypos); \
@@ -518,7 +518,7 @@ blend_##format_name (GstVideoFrame * srcframe, gint xpos, gint ypos, \
     ypos = 0; \
   } \
   /* If x or y offset are larger then the source it's outside of the picture */ \
-  if (xoffset > src_width || yoffset > src_height) { \
+  if (xoffset > src_width || yoffset > src_width) { \
     return; \
   } \
   \
@@ -892,7 +892,7 @@ fill_checker_##name##_c (GstVideoFrame * frame) \
   width = GST_VIDEO_FRAME_WIDTH (frame); \
   width = GST_ROUND_UP_2 (width); \
   height = GST_VIDEO_FRAME_HEIGHT (frame); \
-  dest = GST_VIDEO_FRAME_PLANE_DATA (frame, 0); \
+  dest = GST_VIDEO_FRAME_COMP_DATA (frame, 0); \
   dest_add = GST_VIDEO_FRAME_COMP_STRIDE (frame, 0) - width * 2; \
   width /= 2; \
   \
@@ -922,7 +922,7 @@ fill_color_##name (GstVideoFrame * frame, \
   width = GST_VIDEO_FRAME_WIDTH (frame); \
   width = GST_ROUND_UP_2 (width); \
   height = GST_VIDEO_FRAME_HEIGHT (frame); \
-  dest = GST_VIDEO_FRAME_PLANE_DATA (frame, 0); \
+  dest = GST_VIDEO_FRAME_COMP_DATA (frame, 0); \
   dest_stride = GST_VIDEO_FRAME_COMP_STRIDE (frame, 0); \
   width /= 2; \
   \
