@@ -16,13 +16,12 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /**
  * SECTION:element-osssink
- * @title: osssink
  *
  * This element lets you output sound using the Open Sound System (OSS).
  *
@@ -31,7 +30,8 @@
  * your pipeline works under all circumstances (those conversion elements will
  * act in passthrough-mode if no conversion is necessary).
  *
- * ## Example pipelines
+ * <refsect2>
+ * <title>Example pipelines</title>
  * |[
  * gst-launch-1.0 -v audiotestsrc ! audioconvert ! volume volume=0.1 ! osssink
  * ]| will output a sine wave (continuous beep sound) to your sound card (with
@@ -39,7 +39,7 @@
  * |[
  * gst-launch-1.0 -v filesrc location=music.ogg ! decodebin ! audioconvert ! audioresample ! osssink
  * ]| will play an Ogg/Vorbis audio file and output it using the Open Sound System.
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -66,10 +66,9 @@
 #endif /* HAVE_OSS_INCLUDE_IN_SYS */
 
 #include "common.h"
-#include "gstossaudioelements.h"
 #include "gstosssink.h"
 
-#include <glib/gi18n-lib.h>
+#include <gst/gst-i18n-plugin.h>
 
 GST_DEBUG_CATEGORY_EXTERN (oss_debug);
 #define GST_CAT_DEFAULT oss_debug
@@ -129,8 +128,6 @@ static GstStaticPadTemplate osssink_sink_factory =
 
 #define gst_oss_sink_parent_class parent_class
 G_DEFINE_TYPE (GstOssSink, gst_oss_sink, GST_TYPE_AUDIO_SINK);
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (osssink, "osssink", GST_RANK_SECONDARY,
-    GST_TYPE_OSSSINK, oss_element_init (plugin));
 
 static void
 gst_oss_sink_dispose (GObject * object)
@@ -186,8 +183,8 @@ gst_oss_sink_class_init (GstOssSinkClass * klass)
       "Erik Walthinsen <omega@cse.ogi.edu>, "
       "Wim Taymans <wim.taymans@chello.be>");
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &osssink_sink_factory);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&osssink_sink_factory));
 }
 
 static void
